#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
ctrl_project.py :
   Manage interactions with a luciole Project 
"""

# standard library imports
import os.path
from gettext import gettext as _

# related third party imports
from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable


# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.base.tools as LT
import luciole.constants as LCONST
import luciole.ctrl.constants as CTRL_CONST
import luciole.ctrl.ctrl_acq as CTRL_ACQ
import luciole.ctrl.ctrl_base as CTRL_BASE
import luciole.ctrl.ctrl_timeline as CTRL_TMLN
import luciole.ctrl.ctrl_no_project as CTRL_NO_PROJECT
import luciole.ctrl.ctrl_img_vw as CTRL_IMG_VW
import luciole.ctrl.load_rush as LRUSH
import luciole.gui.constants as GUI_CONST
import luciole.media.image as LI
import luciole.project.project_etree as LPF

from luciole.gui.windows.assistant_new_project import AssistantNewProject
from luciole.ctrl.import_image import ImportController

# NULL tuple constant defined for compat with gobject signals on ctrl.py file
_NULL_TUPLE = None, None

class Project(object) :
    
    
    def __init__(self) :
        self.parser = LPF.ProjectToFile()
        self.props = LPF.ProjectDico()
        self.timeline = None
        self.rush = None
        self.acquirer = None
    
       
    def save(self, path) :
        _project_path  = None 
        if path is not None :
            try :
                _project_path = self.parser.save_as(path , self.props)
            except LEXCEP.LucioException, _err_msg :
                raise LEXCEP.LucioException, _err_msg
        else :
            try :
                _project_path = self.parser.save(self.props)
            except LEXCEP.LucioException, _err_msg :
                raise LEXCEP.LucioException, _err_msg
        return _project_path
    
    def set_non_pjt_dpdt_props(self) :
        # init props not managed by porject in file
        self.props['is_modified'] = False 
        self.props['is-mixer-active'] = False
        self.props['image2mix'] = None
        self.props['alpha'] = LCONST.DEFAULT_ALPHA_IMAGE

class CtrlProject(Signallable, Loggable) :
    
    # CONSTANTS
    MODES = ("NO_PROJECT", "ACQUISITION", "TIMELINE", "IMAGE_VIEW", "EXPORT")
    PROJECT_LOADED_MODES = ("ACQUISITION", "TIMELINE", "IMAGE_VIEW", "EXPORT")
    


    __signals__ = {
            "project-loaded": ["project"],
            "project-closing" : ["project"],
            "project-closed" : [],
            'project-modified' : ["is_modified"],
            'project-mode-changed' : ["mode"],
            'quit' : [],
            'import-image-done' : ['images'], 
            }
    
    def _get_mode(self) :
        """ mode getter """
        return self._mode
    mode = property(_get_mode,
                    None,
                    None,
                    """ project mode """)
    

    def _get_is_modified(self):
        """ is_modified getter """
        return self.project.props['is_modified']

    def _set_is_modified(self, is_modified = True)  :
        """ is_modified getter """
        if isinstance(is_modified, bool)\
                and\
                is_modified != self.project.props['is_modified'] :
            self.project.props['is_modified'] = is_modified
            self.debug('emit signal project-modified : is_modified = %s',
                    is_modified)
            self.emit('project-modified', is_modified)

    is_modified = property(_get_is_modified,
                    _set_is_modified,
                    None,
                    """ is_modified project property """)


    def __init__(self) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        
        self._mode = None
        self.gui = None
        self.project = Project()

        self.ctrl_acq = CTRL_ACQ.ProjectModeAcquisition(self)
        self.ctrl_tmln = CTRL_TMLN.ProjectModeTimeline(self)
        self.ctrl_no_project = CTRL_NO_PROJECT.ProjectModeNoProject(self)
        self.ctrl_img_vw = CTRL_IMG_VW.ProjectModeImgVw(self)
        
        self._connect_to_acq(self.ctrl_acq)
        self._connect_to_timeline(self.ctrl_tmln)


    def connect_to_gui(self, gui) :
        self.gui = gui
        
        self.ctrl_acq.set_viewer(gui.drawarea)
        self.ctrl_tmln.set_viewer(gui.drawarea)
        self.ctrl_no_project.set_viewer(gui.drawarea)
        self.ctrl_img_vw.set_viewer(gui.drawarea)
    
    def no_project_mode(self) :
        self.debug('No project mode')
        
        if self._mode == 'ACQUISITION' :
            self.ctrl_acq.stop()

        if self._mode != 'NO_PROJECT' :
            self._mode = 'NO_PROJECT'
            self.ctrl_no_project.active_viewer()
        else :
            self.warning("impossible to switch to No Project mode (%s)",
                    self._mode )


     

    def acquistion_mode(self) :
        """ activate acquisition mode """
        self.debug('Acquisition mode')
        if self._mode in self.PROJECT_LOADED_MODES \
            and self._mode != 'ACQUISITION'  :
            self._mode = 'ACQUISITION'
            self.ctrl_acq.active_viewer()
            self.ctrl_acq.start()
            self.debug('Emit project-mode-changed : %s'%self._mode)
            self.emit('project-mode-changed',self._mode)

        else :
            self.warning("impossible to switch to acquisition mode (%s)",
                    self._mode )
    
    def timleline_mode(self) :
        """ activate timeline mode """
        if self._mode == 'ACQUISITION' :
            self.ctrl_acq.stop()
        
        if self._mode in self.PROJECT_LOADED_MODES \
            and self._mode != 'TIMELINE'  :

            self._mode = 'TIMELINE'
            self.ctrl_tmln.active_viewer()
            self.debug('Emit project-mode-changed : %s'%self._mode)
            self.emit('project-mode-changed',self._mode)
        else :
            self.warning("impossible to switch to timeline mode")
    

    def img_vw_mode(self, origin, image_name, position = 0) :
        if self._mode in self.PROJECT_LOADED_MODES : 
            if self._mode == 'ACQUISITION' :
                self.ctrl_acq.stop()

            if self._mode == 'TIMELINE' and origin == LCONST.CHRONO :
                self.ctrl_tmln.go_to_position(position)
                # TODO : implemnt better then return here
                return

            if self._mode == 'IMAGE_VIEW' :
                self.ctrl_img_vw.stop()

             
            _factory = self.ctrl_tmln._get_factory(image_name)
            self.ctrl_img_vw.active_viewer(_factory)
            self._mode = 'IMAGE_VIEW'
     
    def open(self, project_path = None) :
        """ open an existing project """
        # no project path  : leave
        if project_path == None :
            return
        self.debug("Starting project load of %s", project_path)

        # if a project is loaded close it
        if self._mode != 'NO_PROJECT' :
            # (None, None) params for compat with gobject signal
            # TODO : manage close project
            #self.__cbs['close-project'](None, None)
            pass
        
        self.project = Project()
        try :
            (_is_valid, self.project.props) = \
                    self.project.parser.open(project_path)
        except LEXCEP.LucioException, _err_msg :
            # Transmit error to controller: not stop application
            self.error(_err_msg)
            # TODO : manage exceptions
            raise LEXCEP.LucioException, _err_msg
        else :
            if not _is_valid :
                _msg = _(" Webcam data not valid in project.\
                        Please restart webcam detection")
                self.gui.windows.error_message( _msg)
            
            # initiate project load
            self._start_project_load()
    
    def new_project(self ) :
        """ new project creation """
        if self._mode != 'NO_PROJECT' :
            # close current project
            self.close()
            
        _ass = AssistantNewProject(self._new_project_cb)
        
        # TODO : fot test purpose only
        import os.path
        import shutil
        PROJECT_NAME = 'test_luciole'
        PROJECT_PATH = '/home/nico/temp/testLuciole'
        
        _p_path = os.path.join(PROJECT_PATH, PROJECT_NAME)
        if os.path.exists(_p_path) :
            # delete folder
            self.debug('Deleting folder %s', _p_path)
            shutil.rmtree(_p_path)

        # set on assistant project name
        _ass._pg1.entry_project_name.set_text(PROJECT_NAME)
        # set a folder
        _ass._pg1._file_chooser.set_current_folder(PROJECT_PATH)

        
        
    def _new_project_cb(self, project_data) :
        """ 
        new project callback : 
        signal generated by new project assistant
        """
        self.debug(" create project with %s", project_data)
        if project_data is None :
            self.error('No data to load')
            return
        # create project on disk
        self.project = Project()
        try :
            _path = self.project.parser.create(project_data)
        except LEXCEP.LucioException, _err :
            raise LEXCEP.LucioException, _err
        
        self.debug(" created project on disk : %s", _path)
        # load project on luciole
        self.open(_path)

            
    def close(self) :
        """ close current project """
        if self._mode in self.PROJECT_LOADED_MODES :
            if self.project.props['is_modified'] == True :
                if (self.emit('project-closing', self.project) == True) :
                    self.save()
            
            self.debug(" Closing current project ")
            self.release()
            self.no_project_mode()
            self.debug('emit signal project-closed')
            self.emit('project-closed')
            self.project = None
    
    def save(self, dir_path = None ) :
        if self._mode not in self.PROJECT_LOADED_MODES :
            self.error('invalid mode for save : %s'%self._mode)
            return
        if dir_path and not os.path.exists(dir_path) :
            self.error('Path %s exists. Impossible to save project', dir_path)
            return
        self.debug("%s",dir_path)  
        try :
            _project_path = self.project.save(dir_path)
        except LEXCEP.LucioException, _err_msg :
            self.error('%s', _err_msg)
        else : 
            self.debug('Project saved in %s', _project_path)
            self.is_modified = False
            if dir_path is not None and _project_path is not None  :
                # reload project is corretcly saved
                self.close()
                self.debug('Opening project with path %s',_project_path)
                self.open(_project_path)
    
    def set_framerate(self, fpi, force = False) :
        if self._mode != "TIMELINE" and force == False :
            self.info('update of fpi only availabe in TIMELINE mode')
            return
        if int(self.project.props['fpi']) != fpi :
            self.project.props['fpi'] = str(fpi)
            self.is_modified = True
            # update timeline 
            self.ctrl_tmln.update_image_duration(fpi)
        


    def new(self, project_data = None) :
        
        # create new project

        if  project_data :
            # TODO case with assistant
            pass



    def release(self) :

        # release signals between aquirer and project
        if self.project.props['hardtype'] in \
                (LCONST.FAKE, LCONST.DVCAM, LCONST.WEBCAM ) : 
            #self._disconnect_from_acq(self.ctrl_acq)
            pass

        # release signals between timeline and project
        # self._disconnect_from_timeline(self.ctrl_tmln)
        
        # disconnect from gui
        # TODO : Analyse when to be done on close project or quit ?
        #self.disconnect_from_gui(self.gui)


    def import_images(self, paths) :
        if paths != [] :
            self.debug('Starting image importer for %s',paths)
            importer =  ImportController(   paths,
                                        self.project.props,
                                        self.project.rush,
                                        self.gui.status_bar
                                        )
            importer.connect('import-image-done', self._import_img_done_cb)
        else :
            self.info('No image to import')

    def  _import_img_done_cb(self, imager, images) :
        """ import-image-done callback """
        self.debug("RX import-image-done : %s", images)
        if images != [] :
            self.emit('import-image-done', images)
            
            # update project 
            # get image names
            _names = [ _img.name for _img in images]
            self.project.props['rush_images'].extend(_names)
            self.project.props['capture_images'].extend(_names)
            self.is_modified = True
        else :
            self.warning('No image imported')


    
    def _start_project_load(self) :
        
        
        # start rusher controller
        _rush_ctrl = LRUSH.ControllerLoadRush(   self.project,
                                    self.gui.status_bar )
        self._connect_to_rush(_rush_ctrl)
        
        # start acquirer controller
        if self.project.props['hardtype'] in \
            (LCONST.FAKE, LCONST.DVCAM, LCONST.WEBCAM ) :  
            # start acquisition controller
            #self.acq_ctrl = CTRL_ACQ.CtrlAcq(self)
            self.debug('Acquirer Object loaded %s', self.ctrl_acq)
        else :
            # No acquisition (only import and timeline)
            self._mode = "TIMELINE"

        # start timeline controller
        #self.ctrl_tmln = CTRL_TMLN.CtrlTimeline(self)


    def _connect_to_rush(self, rush_ctrl) :
        rush_ctrl.connect('rush-loaded', self._rush_load_finish_cb)

    def _disconnect_from_rush(self, rush_ctrl) :
        rush_ctrl.disconnect_by_function( self._rush_load_finish_cb)

    def _rush_load_finish_cb(self, rush_ctrl, rush_obj) :
        self.debug(" Rush Load Finish :%s ", rush_obj.dump_image_name() )
        if rush_obj != None : 
            self._terminate_project_load(rush_obj)
        # Load rush is done only one time at load so disconnect it
        self._disconnect_from_rush(rush_ctrl)
    
    def _terminate_project_load(self, rush_obj) :
        self.project.rush = rush_obj
        
        # set project as not modified as it is just loaded
        self.project.set_non_pjt_dpdt_props()
        
        # set path for image to mix
        self.project.props['image2mix'] =\
                os.path.join(
                        self.project.props['project_dir'],
                        LCONST.IMAGE2MIX_NAME
                        )

        # if a capture image exist ,
        # mix with last one by copying it in 'image2mix'
        
        if self.project.props['capture_images'] != []:  
            self.debug(' image 2 copy %s', 
                    self.project.props['capture_images'][-1])
            self._copy_to_image2mix(
                    self.project.props['capture_images'][-1])             

        self.debug("emit project-loaded : %s " , self.project) 
        self.emit('project-loaded', self.project )
    
    def _connect_to_acq(self , ctrl_acq) :
        ctrl_acq.connect('acquirer-ready', self._acquirer_ready_cb)
        ctrl_acq.connect('acquirer-error', self._acquirer_error_cb)

    def  _disconnect_from_acq(self , ctrl_acq) :
        ctrl_acq.disconnect_by_function( self._acquirer_ready_cb)
        ctrl_acq.disconnect_by_function( self._acquirer_error_cb)


    def _acquirer_ready_cb(self, ctrl_acq ) :
        self.debug('RX acquirer-ready signal')
        # TODO : work around on mode
        # FIXME : set to image to force transtion to acquisition mode
        # default mode when project is started
        self._mode = 'IMAGE_VIEW'
        self.acquistion_mode()



    def _acquirer_error_cb(self, ctrl_acq, msg) :
        self.error(msg)
    
    def _connect_to_timeline(self, ctrl_tmln) :
        ctrl_tmln.connect('timeline-loaded', self._timeline_loaded_cb)
        
    def _disconnect_from_timeline(self, ctrl_tmln) :
        ctrl_tmln.disconnect_from_project(self)
        ctrl_tmln.disconnect_by_function(self._timeline_loaded_cb)

    def _timeline_loaded_cb(self, ctrl_tmln): 
        pass
        #self.project.timeline = timeline
    
    
    def _copy_to_image2mix(self, image_name) :
        
        _source = os.path.join(     self.project.props['project_dir'],
                                        self.project.props['rush_dir'],
                                        image_name)
        try :    
            LT.copyf(_source, self.project.props['image2mix'])  
        except LEXCEP.LucioException, err :
            self.error('impossible to copy %s in %s -- %s ',
                    _source,
                    self.project.props['image2mix'],
                    err)
        else :
            self.debug(' copied %s in %s',
                    _source,
                    self.project.props['image2mix'])




