#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
ctrl_acq.py :
    manages acquisition
"""
# standard library imports
import os.path
import urllib
# related third party imports
import gst
import copy

from pitivi.signalgroup import SignalGroup
from pitivi.pipeline import Pipeline
from pitivi.action import ViewAction
from pitivi.timeline.timeline import Timeline, TimelineObject, TimelineError

from pitivi.factories.timeline import TimelineSourceFactory

from pitivi.stream import AudioStream, VideoStream
from pitivi.settings import GlobalSettings
from pitivi.settings import ExportSettings
from pitivi.sourcelist import SourceList
from pitivi.timeline.track import Track, SourceTrackObject

from pitivi.utils import Seeker, getNextObject
# local application/library specific imports
from luciole.ctrl.base import ProjectMode
import luciole.base.tools as LT
import luciole.constants as LCONST
import luciole.base.exceptions as LEXCEP

class LucioleTimeline(Timeline) :
    DEFAULT_DURATION = 1 * gst.SECOND

    def __init__(self) :
        Timeline.__init__(self)
        # TODO : Temporary usage of GlobalSettings
        self.settings = GlobalSettings()
        self.default_duration = self.DEFAULT_DURATION
        
    def change_order(self, images_list, position_dest) :
        self.info("order changed move images %s at position %s", 
                images_list, 
                position_dest)
        
        _tl_objects_to_move = [self.video_objects[_elem[0]] for _elem in images_list]
        self.debug( " Objs to move :%s", [ obj.factory.name for obj in _tl_objects_to_move])

        if position_dest[0] < len(self.video_objects ) :
            # Move images inside sequence
            _tl_obj_dest = self.video_objects[position_dest[0]]
        
            for tl_obj  in _tl_objects_to_move :

                self.info(" Moving image %s (position %s) before %s (position : %s)",\
                    tl_obj.factory.name,
                    self.video_objects.index(tl_obj),
                    _tl_obj_dest.factory.name,
                    self.video_objects.index(_tl_obj_dest))

                _tl_obj_tmp = self._move_timeline_object(tl_obj,_tl_obj_dest)
                _tl_obj_dest = \
                    self.video_objects\
                    [self.video_objects.index(_tl_obj_tmp) + 1]
        
        else :
            # move images at the end of sequnce
            for tl_obj  in _tl_objects_to_move :
                self.info(" Moving image %s (position %s) at end of sequence",\
                    tl_obj.factory.name,
                    self.video_objects.index(tl_obj) )
                self._move_at_end(tl_obj)
        
       
    def insert_factories(self, factories, position_dest) :
        self.info("images to insert %s at position %s", 
                factories, position_dest)
        
        if position_dest[0] < len(self.video_objects ) :
            self._insert_factories_at_position( position_dest[0], factories)
        else :
            for _factory in factories :
                # TODO : verify if append merhod usage is correct. regarding 
                # _insert_factories_at_position method (i.e. tl_obj creation and
                # disable/enable timeleine methods) 
                self._append_obj(_factory)
        
        

    
    def remove_images(self, images_list) :
        self.info("images to remove on timeline %s", images_list)
        _tl_objs_to_remove = \
                [self.video_objects[_elem[0]] for _elem in images_list]
        
        for tl_obj in _tl_objs_to_remove :
            self._remove_object(tl_obj)
        
         
  

    
    def create_timeline_object(self, factory) :
        
        _stream_map = self._getSourceFactoryStreamMap(factory)
        _timeline_object = TimelineObject(factory)
        for stream, track in _stream_map.iteritems():
            self.info("Stream: %s, Track: %s, Track duration: %d", str(stream),
                       str(track), track.duration)
            track_object = SourceTrackObject(factory, stream)
            track.addTrackObject(track_object)
            _timeline_object.addTrackObject(track_object)
        _timeline_object.start = 0
        return _timeline_object




    
    def add_sound_track(self, audio_factory, duration, is_video_duration = True,   audio_offset = 0,) :
        # TODO : only a offset of 0 is vlaid. with ofsset !=0 --> pieleine error
        audio_offset = 0
        # TODO : undestand better disable/ Enable updates.
        # the usage od disable/enable avoid a fade transition at start ?
        _vtrack = self._get_video_track()
        self.debug('Video track duration %s, ', _vtrack.duration)
        if is_video_duration :
            _duration = _vtrack.duration
        else :
            _duration = duration
      
        self.debug('Sound track %s, duration %s, audio offest %s', 
                audio_factory.name, _duration, audio_offset)
        
        self.disableUpdates() 
        _tl_obj = self.create_timeline_object(audio_factory)
        _tl_obj.start = audio_offset
        _tl_obj.duration = _duration
        self.addTimelineObject( _tl_obj)
        self.enableUpdates()
        return _tl_obj
    
    def remove_sound_track(self, factory) :
        # remove factory from timeline
        self.removeFactory(factory)
        # update duration
        _vtrack = self._get_video_track()
        if _vtrack.duration != self.duration :
            self.duration = _vtrack.duration



            


        


    def update_timeline_duration(self, duration, sound_stop_with_video = True ) :
        
        
        # copy objects for vaoiding update problems
        _video_objs = copy.copy(self.video_objects)
        self.debug('New duration %s image to update %s', duration, [ _obj.factory.name for _obj in _video_objs ] )
        
        _old_duration = _video_objs[0].duration
        # check type of image duration update
        # if duration update increase, start update by the last image 
        #    in timeline (right to left order)
        # if duration update decrease, start update fu the first image
        #    in timeline (left to riht order )
        
        _reverse = False
        if duration > _old_duration :
            # new duration > old duration
            # start update of images in reverse order
            _start = (len(_video_objs) -1) * duration 
            _video_objs.reverse()
            _reverse = True
        else :
            # new duration < old duration
            _start = 0

        for _obj in _video_objs :
             
            self.debug('Before Update %s : start %s duration %s, factory duration %s', 
                    _obj.factory.name,
                    _obj.start,
                    _obj.duration,
                    _obj.factory.duration)
               
            _obj.setDuration(duration, True)
            _obj.setStart(_start, True) 
            if _reverse == True :
                _start = _start - duration
            else :
                _start = _start + duration
            
            self.debug('Updated %s : start %s duration %s ', 
                    _obj.factory.name,
                    _obj._getStart(),
                    _obj._getDuration())
        
        _total_duration = len(_video_objs) * duration 
        self.debug(' Total duration %s', _total_duration)
        if sound_stop_with_video == True :
            _obj = self._get_audio_object()
            #TODO ; check case of audio track duration < images duration
            if _obj is not None : 
                _obj.setDuration(_total_duration, True, True)
                self.debug('NEW audio %s track duration %s factory duration %s',_obj, _obj.duration, _obj.factory.duration)
        self.debug('NEW timeline duration %s', self.duration )

    
    def _move_timeline_object(self, tl_src, tl_dest) :
        _idx_src = self.video_objects.index(tl_src)
        _idx_dest = self.video_objects.index(tl_dest)
        # select objects to shift on timeline 
        if _idx_src < _idx_dest :
            _objs_to_move = self.video_objects[_idx_src+1:_idx_dest]
            _duration = -1 * tl_src.duration
            #set start duration of src before dest
            tl_src.start = self.video_objects[_idx_dest -1].start
        else :
            _objs_to_move = self.video_objects[_idx_dest:_idx_src]
            _duration = 1 * tl_src.duration
            #set start duration at  dest
            tl_src.start = self.video_objects[_idx_dest].start

        # shift objects to move
        self._shift_object(_objs_to_move, _duration)

        return tl_src 
    
    def _move_at_end(self, tl_src):

        # get objecs to move 
        _objs_to_move = self._getVideoObjsAfter(tl_src)

        # set last postion fo tl_src
        tl_src.start = self.video_objects[-1].start
        
        # shift back the objects to move
        _duration = -1*tl_src.duration
        self._shift_object(_objs_to_move, _duration)
     
    def _move_object(self, pos_src, pos_dest) :
 
        if pos_src == pos_dest :
            # nothing to move
            return
        _pos_dest = self.video_objects[pos_dest].start
        _obj_src = self.video_objects[pos_src]
        # select objects to move
        if pos_src > pos_dest :
            _objs_to_move = self.video_objects[pos_dest:pos_src]
            _duration = 1 * _obj_src.duration
        else :
            _objs_to_move = self.video_objects[pos_src+1:pos_dest+1]
            _duration = -1 * _obj_src.duration
        # shift objects to move
        self._shift_object(_objs_to_move, _duration)
        
        # move object at despoistion
        _obj_src.start = _pos_dest
    
    def _get_track_objects(self, objs, track) :
        track_objs = []
        for _obj in objs :
            if track == _obj.track_objects[0].track :
                track_objs.append(_obj)

        return track_objs





    def _insert_factories_at_position(self, position, factories_to_insert) :
        # TODO : undestand better disable/ Enable updates.
        # the usage od disable/enable avoid a fade transition between images: Why ?
        self.disableUpdates() 
        _timeline_objects = []
        _position_obj = self.video_objects[position]
        self.debug( "_position_obj :%s", _position_obj.factory.name)
        _objs_after = self._getVideoObjsAfter(_position_obj) 
        _objs_after.insert(0, _position_obj)
        
        _dur = _position_obj.start
        
        for _factory in factories_to_insert :
            _timeline_obect = self.create_timeline_object(_factory)
            _timeline_objects.append(_timeline_obect)
        
        _insert_duration = self._calc_objs_duration(_timeline_objects)
        self._shift_object(_objs_after, _insert_duration)
        
        for  _tl_obj in _timeline_objects :
            _tl_obj.start =  _dur
            _dur += _tl_obj.duration
            self.addTimelineObject( _tl_obj)
        
        self.enableUpdates()
  
    def _calc_objs_duration(self, objs_to_insert) :
        _duration = 0
        for _obj in objs_to_insert :
            _duration += _obj.duration
        self.info("shift duration = %s", _duration)
        return _duration
        
    def _shift_object(self, timeline_objects, duration) :
        for _obj in timeline_objects :
            self.debug("%s", _obj.factory.name)
            _obj.start += duration
        self.info ( "%s timeline objects shifted from %s"%(len(timeline_objects), duration))

    def _append_obj(self, factory):
    
        timeline_object = self.addSourceFactory(factory)
        self.info("appended  %s"%timeline_object)
        return timeline_object
    
    def _remove_object(self, tl_obj) :
        _objs_after = self._getVideoObjsAfter(tl_obj)
        _duration = -1* tl_obj.duration
        self.debug("obj to remove %s ", tl_obj)  
        _vtrack = self._get_video_track()
        _vtrack.removeTrackObject(tl_obj)
        
        # update start point of objects after deleted one
        self._shift_object(_objs_after, _duration)

   
    def _get_video_track(self) :
        for track in self.tracks:
            if isinstance(track.stream, VideoStream) :
                return track

    def _get_audio_track(self):
        for track in self.tracks:
            if isinstance(track.stream, AudioStream) :
                return track

    
    def _get_video_track_objects(self) :
        _vtrack = self._get_video_track()
        
        _res = _vtrack.track_objects
        #self.debug("Video Track objects : %s", 
        #        [ obj.factory.name for obj in _res])

        return _res
    video_objects = property(_get_video_track_objects)

    def _get_audio_object(self) :
        _audio_obj = []
        _res = None
        for _obj in self.timeline_objects :
            if isinstance(_obj.track_objects[0].stream, AudioStream) :
                _audio_obj.append(_obj)

        if len(_audio_obj) > 1 :
            raise TimelineError(" More then one audio stream in timeline")
        elif len(_audio_obj) == 1 :
            _res = _audio_obj[0]
            

        return _res
    audio_object = property(_get_audio_object)


    def _getVideoObjsAfter(self, timeline_object) :
        """ 
        get video objects after timeline objects 
        This function retrieves objects only from video track
        """
        objects = []
        priority= None
        if timeline_object in self.video_objects :
            _res = getNextObject(timeline_object, self.video_objects, priority) 
            while _res is not None :
                objects.append(_res)
                _res = getNextObject(_res, self.video_objects, priority) 
        else :
            self.error('timeline_object %s not in video_track', 
                timeline_object.factory.name)  
        
        self.debug(" video objs after %s : %s", 
                timeline_object.factory.name,
                [ obj.factory.name for obj in objects])
        return objects

    def _get_image_sequence(self) :
        _list = [ _obj.factory.name for _obj in  self.video_objects] 
        return _list
            
  


class ProjectModeTimeline(ProjectMode):
    __signals__ = {
            'timeline-loaded' : ['timeline'] ,
            'sound-props-changed' : ['active', 'stop_with_video'],
            }
    
    _25_FPS = (1 * gst.SECOND)/25

    def __init__(self, ctrl_project) :
        ProjectMode.__init__(self, ctrl_project, "TIMELINE") 
        
        self.acq_signals = SignalGroup()
        self.timeline_signals =  SignalGroup()
        self.debug('ProjectModeTimeline initialized ')
        self._is_stop_sound_with_video = False
        
        # prepare connection to acquisiton mode
        self._connect_to_ctrl_acq(self.ctrl_project.ctrl_acq)

    def prepare(self, project) :
        ProjectMode.prepare(self, project)
        
        # prepare sources 
        self.sources = SourceList()
        self.sources.connect("source-added", self._sourceAddedCb)
        self.sources.connect("source-removed", self._sourceRemovedCb)
        self.uris = []

        

        # prepare timeline
        self.timeline = LucioleTimeline()
        self._timeline_factory = TimelineSourceFactory(self.timeline)
        
        # prepare pipeline and resource 
        self._pipeline = Pipeline()
        # TODO : connect to pipeline signals ?
        #self._pipeline.connect('eos', self._eos_cb)

        self._view_action = ViewAction()
        self._view_action.addProducers(self._timeline_factory)
        self._pipeline.addAction(self._view_action)
        # why seeker ? 
        self.seeker = Seeker(80)
        
        #
        # initialize video settings
        #
        settings = self.getAutoSettings()
        self._videocaps = settings.getVideoCaps()

        # fill timeline with audio and video track
        video = VideoStream(gst.Caps(settings.getVideoCaps()))
        track = Track(video)
        self.timeline.addTrack(track)
        audio = AudioStream(gst.Caps(settings.getAudioCaps()))
        track = Track(audio)
        self.timeline.addTrack(track)

        
        self._load_timeline(self.project)
        
        self.debug("Prepare Timeline : %s", self.timeline )
        
            
    def release(self) :
        self._mode = "INVALID"
        self._pipeline.release()
        self._pipeline = None
        del self._timeline_factory
        del self.timeline
        self._view_action = None



    def active_viewer(self) :
        if self._viewer :
            self.debug(" Timeline :%s, timeline duration :%s ", self.timeline, self.timeline.duration)
        
            self._viewer.setPipeline(None)
            self._viewer.showSlider()
            self._viewer.setAction(self._view_action)
            self._viewer.setPipeline(self._pipeline, "TIMELINE")
            
            # chnage pipeline status to activate it ( seek, duration)
            self._pipeline.pause()
            self.debug("Pipeline state : %s", self._pipeline.getState())
            self._viewer.prepare_viewer(self.timeline.duration)
            # FIXME : hack to make the gtk.HScale seek slider UI behave properly
            #self._viewer._durationChangedCb(None, self.timeline.duration) 
            self.debug("actual sequence %s", [_obj.factory.name for _obj in self.timeline.video_objects])
        
        else :
            self.error("No viewer defined")
    
    def change_order(self, images, target) :
        self.debug('images : %s , target :%s', images, target)
        self.timeline.change_order(images, target)
        # update project
        self._update_project()
        
    def insert(self, images, target) :
        self.debug('images : %s , target :%s', images, target)
        # get factories from source
        _factories = [ self._get_factory(_elem) for _elem in images ]
        # insert factories on timeline
        self.timeline.insert_factories(_factories, target)
        # update project
        self._update_project()

    def remove(self, images) :
        self.debug('images : %s'  , images)
        self.timeline.remove_images(images)
 
        # update project
        self._update_project()

    def _update_project(self) :
        """ update project properties according image sequence """ 
        _img_seq = self.timeline._get_image_sequence()
        self.info('Image sequence : %s', _img_seq)
        # update project according image sequence
        self.project.props['chrono_images'] =_img_seq
        self.ctrl_project.is_modified = True


    def add_sound_track(self, path) :
        self.debug('Add sound track : %s', path )
        # check if a sound track is avlbl, if yes remove it
        _dest_path = os.path.join(   self.project.props['project_dir'],
                                LCONST.SOUND_TRACK_NAME)

        # first remove current sound track from timeline and sources
        self._remove_sound_track(True)
             

        
        # copy sound track on project
        # TODO : allow only ogg files
        
        try :
            LT.copyf(path, _dest_path) 
        except LEXCEP.LucioException, err:
            self.error(err)
        else :
            self.AddSources([_dest_path])
            self.ctrl_project.is_modified = True
            self.project.props['sound']['name'] = LCONST.SOUND_TRACK_NAME
            self._update_sound_props(self.project.props, acive='no')

    
    def go_to_position(self, position) :
        
        self._viewer.seek(position*self.default_duration)
    
    def update_image_duration(self , fpi ) :
        self.default_duration =   self._25_FPS * fpi
        # upadte image factory duration
        for _factory in self.sources.getSources() :
            if _factory.name == LCONST.SOUND_TRACK_NAME :
                # dont update sound track duration
                continue
            _factory.duration =  self.default_duration
            self.debug('Updated duration (%s) of factory %s',
                    _factory.duration,
                    _factory.name)

        
        self.timeline.update_timeline_duration(self.default_duration, self._is_stop_sound_with_video)
        
       

    def _remove_sound_track(self, force = True) :
        """ 
        if force = True --> rm from timeline and source list
        if force = False --> rm from timeline only
        """
        _factory = self._get_factory(LCONST.SOUND_TRACK_NAME)
        if _factory is None :
            # nothing to suppres
            return

        if self.project.props['sound']['active'] == 'yes' :
            self.timeline.remove_sound_track(_factory)
            # disconnect callback who updates audio duration regarding
            # video duration
            try :
                self.timeline.disconnect_by_function(self._video_track_added)
            except :
                #TODO : what we do ?
                pass
        if force == True :
            self.sources.removeUri(_factory.uri)
        
        self._is_stop_sound_with_video = False

                    
            
    
    def add_sound_track_to_timeline(self) :
        _factory = self._get_factory(LCONST.SOUND_TRACK_NAME)
        if _factory : 
            self.debug('Sound Factory %s, %r , duration %s', _factory.name, _factory, _factory.duration)
            _tl_obj = self.timeline.add_sound_track(    _factory,
                                                    _factory.duration,
                                                    self._is_stop_sound_with_video,
                                                    0)
            self.timeline.connect('timeline-object-added', self._video_track_added, _tl_obj)
        else :
            self.warning ("Sound factory does not exists")
        
        self._update_sound_props(self.project.props, active ='yes')




    
    def remove_sound_track_from_timeline(self) :
        _factory = self._get_factory(LCONST.SOUND_TRACK_NAME)
        if _factory :
            self.debug('remove Sound Factory %s from timeline', _factory.name)
            self.timeline.removeFactory(_factory)
            self.timeline.disconnect_by_function(self._video_track_added)
        else :
            self.info('No sound track to remove')
        
        self._update_sound_props(self.project.props, active ='no')

    
    
    def _video_track_added(self, timeline , timeline_object , audio_object) :
        if timeline_object.track_objects[0] in timeline.video_objects :
            # a video object is added
            _vtrack = timeline._get_video_track()
            self.debug('video track duration %s , object added %s , audio_object duration :%s ',
            _vtrack.duration, timeline_object.factory.name, audio_object.duration)
            
            if _vtrack.duration >  audio_object.duration :
                audio_object.duration = _vtrack.duration
                self.debug ('Audio object duration updated')
        else :
            self.info('timeline _object %s not a video track', timeline_object.factory.name)
    
    def  stop_sound_with_video(self, is_stop = True) :
        self._is_stop_sound_with_video = is_stop
        # check if audio_track yet in timeline
        try :
            _obj = self.timeline.audio_object
        except TimelineError, err :
            self.error('Timeline error : %s',err)
        else :
            self.debug("Update duration")
            if _obj :
                if self._is_stop_sound_with_video :
                    _vtrack = self.timeline._get_video_track()
                    _duration = _vtrack.duration
                    _stop_with_video = 'yes'

                else :
                    _factory = self._get_factory(LCONST.SOUND_TRACK_NAME)
                    _duration = _factory.duration
                    _stop_with_video = 'no'
                
                self._update_sound_props(self.project.props, 
                                    stop_with_video = _stop_with_video)
                _obj.duration = _duration
                self.debug("timeline duration %s", self.timeline.duration)
        


    def _connect_to_ctrl_acq(self, ctrl_acq) : 
        self.debug(" Connecting to ProjectModeAcquisition ")
        self.acq_signals.connect(
                ctrl_acq,'snapshot-taken', None,  self._snapshot_taken_cb)

    
    def _snapshot_taken_cb(self, ctrl_acq, rush_image) :
        self.debug('RX signal snapshot-taken : %s', rush_image.name)
        #TODO : set the correct duration
        self.AddSources([rush_image.path] ) 
    
    def _load_timeline(self, project) :
        _project = project.props
        self.debug('rush images to load : %s', _project['capture_images'] ) 
        _sources = \
            [ os.path.join( _project['project_dir'], 
                            _project['rush_dir'], 
                            _image) \
                    for _image in _project['capture_images'] ]
        _image_duration = self._25_FPS *  int(_project['fpi'])
        uris = ["file://" + urllib.quote(os.path.abspath(path))\
                for path in _sources]
        
        # check if sound track active
        if _project['sound']['active'] == 'yes' :
            # check if sound track really present
            _sound_track =  os.path.join(
                                _project['project_dir'], 
                                LCONST.SOUND_TRACK_NAME)
            if os.path.exists(_sound_track) :
                _uri =  "file://" + urllib.quote(os.path.abspath(_sound_track))
                uris.append(_uri)
            else :
                self.error(
                    'Sound track active but no sound file present (%s)' %
                     _sound_track)

        # discover the factories
        closure = {"rediscovered": 0}
        discoverer = self.sources.discoverer
        discoverer.connect("discovery-done", self._discovererDiscoveryDoneCb,
                _project, _image_duration, uris, closure)
        discoverer.connect("discovery-error", self._discovererDiscoveryErrorCb,
                _project, _image_duration, uris, closure)
         
        # start discoverer

        self.sources.addUris(uris)
        self.default_duration = _image_duration

    
    def _finishLoadingProject(self, project, duration) :
        _start = 0 
        self.debug ('project %s', project)
        for _image in project['chrono_images'] :
            _factory = self._get_factory(_image)
            _factory.default_duration = duration
            _tl_obj = self.timeline.create_timeline_object(_factory)
            _tl_obj.start = _start
            _tl_obj.duration = duration
            _start += duration
            self.timeline.addTimelineObject( _tl_obj)
        if  project['sound']['active'] == 'yes' :
            _factory = self._get_factory(LCONST.SOUND_TRACK_NAME)
            if _factory : 
                
                self.debug('Sound Factory %s, %r , duration %s', 
                        _factory.name, 
                        _factory, 
                        _factory.duration)
                self._is_stop_sound_with_video = False
                if project['sound']['stop_with_video'] == 'yes' :
                    self._is_stop_sound_with_video = True
                _tl_obj = self.timeline.add_sound_track(
                        _factory,
                        _factory.duration,
                        self._is_stop_sound_with_video,
                        0)
                
                self.timeline.connect('timeline-object-added', self._video_track_added, _tl_obj)
        # force emit of project sound props
        self._update_sound_props(
                project = project, 
                active = project['sound']['active'],                
                stop_with_video =  project['sound']['stop_with_video'])


        self.debug('Emit timeline-loaded')
        self.emit('timeline-loaded')
    

       
    def _update_sound_props(self, project, active = None, stop_with_video = None) :
        _is_modified = False
        _is_active = None
        _is_stop_with_video = None
        if project is None :
            self.error("No project defined") 
            return
        
        if active is not None :
            if active not in ('yes','no') :
                self.error('invalid value for prop active (%s)',
                        active)
                return
            if active != project['sound']['active'] :
                project['sound']['active'] = active
                _is_modified = True
            
            if active == 'yes' :
                _is_active = True
            else :
                _is_active = False
        if stop_with_video :
            if stop_with_video not in ('yes','no') :
                self.error('invalid value for prop active (%s)', 
                        stop_with_video)
                return

            if stop_with_video != project['sound']['stop_with_video'] :
                project['sound']['stop_with_video'] = stop_with_video
                _is_modified = True
            if stop_with_video == 'yes' :
                _is_stop_with_video = True
            else :
                _is_stop_with_video = False

        self.debug('sound is_active :%s is_stop_with_video : %s, project %s',
            active,
            stop_with_video,
            project['sound'] )
        
        if _is_modified == True :
            self.ctrl_project.is_modified = True
        
        if  _is_stop_with_video is not None \
            or \
            _is_active is not None :   
            self.debug('Emit sound-props-changed : active %s stop_with_video %s',
                   _is_active, _is_stop_with_video  )
            self.emit('sound-props-changed', _is_active, _is_stop_with_video)


    
    # Sources associated functions 
    #
    def AddSources(self, file_list) :

        uris = ["file://" + urllib.quote(os.path.abspath(path))\
                for path in file_list]
        self.uris = uris 
        self.debug("Sources to add %s", uris) 
        # add uris
        self.sources.addUris(uris)

    def _discovererDiscoveryDoneCb(self, discoverer, uri, factory,
            project , duration, uris, closure):
        self.debug("discovery done for %s",factory.name)
        if factory.uri not in uris:
            # someone else is using discoverer, this signal isn't for us
            return
        
        # update duration of image factory
        if factory.name != LCONST.SOUND_TRACK_NAME :
            factory.duration = self.default_duration


        if factory not in self.sources.getSources() :
            self.debug("Factory duration :%s", factory.duration)
            self.sources.addFactory(factory)
        
        closure["rediscovered"] += 1
        if closure["rediscovered"] == len(uris):
            self._finishLoadingProject(project, duration)

    def _discovererDiscoveryErrorCb(self, discoverer, uri, factory,
            project, duration, uris, closure):
        self.warning('Impossible to load %s'%uri)
    
    def _get_factory(self, image_name) :
        _factory = None
        for _source in self.sources.getSources() :
            if _source.name == image_name :
                _factory = _source
                break
        if _factory == None :
            self.error("No factory found for %s", image_name)
        return _factory

    def _sourceAddedCb(self, sourcelist, factory):
        #self.info("source added to timeline : %s %s"%(sourcelist, factory))
        factory.setFilterCaps(self._videocaps)
        factory.default_duration = self.default_duration
        #        #timeline_object = self.timeline.addSourceFactory(factory)
        #self.info("Object %s starts at %s", factory.name, timeline_object.start)
        self.debug("RX source-added Source %s added -- %s", factory.name, factory)

    def _sourceRemovedCb(self, sourcelist, uri, factory):
        self.timeline.removeFactory(factory)
        self.info('Source %s removed call back'%uri) 
        # TODO : emit source-removed ? 
        #self.emit('source-removed', uri, factory)

    def getAutoSettings(self):
        """
        Computes and returns smart settings for the project.
        If the project only has one source, it will be that source's settings.
        If it has more than one, it will return the largest setting that suits
        all contained sources.
        """
        settings = ExportSettings()
        if not self.timeline:
            self.warning\
            ("project doesn't have a timeline, returning default settings")
            return settings

        # FIXME: this is ugly, but rendering for now assumes at most one audio
        # and one video tracks
        have_audio = have_video = False
        for track in self.timeline.tracks:
            if isinstance(track.stream, VideoStream) and track.duration != 0:
                have_video = True
            elif isinstance(track.stream, AudioStream) and track.duration != 0:
                have_audio = True

        if not have_audio:
            settings.aencoder = None

        if not have_video:
            settings.vencoder = None

        return settings

