#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
actions.py :
   Manage 
"""
# standard library imports
from gettext import gettext as _

# related third party imports
import gobject
import gtk
from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable

# local application/library specific imports
import luciole.gui.constants as GUI_CONST
import luciole.constants as LCONST

_tv_change_signal = 'treeview-change'
_tv_selected_signal = 'treeview-selected'

class TreeviewAction(gtk.Action) :
       
    def __init__(self, name, label = None, tooltip = None, stock_id = None ) :
        gtk.Action.__init__(self, name, label, tooltip, stock_id)
        self._proxies = [] 
        

    def connect_proxy(self, proxy) :
        proxy.connect(_tv_change_signal, self._treeview_change_cb)
        proxy.connect(_tv_selected_signal, self._treeview_selected_cb)
        self._proxies.append(proxy)

    def get_proxies(self) :
        return self._proxies
    
    def _treeview_change_cb(self, widget, data) :
        self.emit(_tv_change_signal, data)
        
    def _treeview_selected_cb(self, widget, data) :
        self.emit(_tv_selected_signal, data)

gobject.signal_new(_tv_change_signal, TreeviewAction,
                       gobject.SIGNAL_RUN_LAST,
                       gobject.TYPE_NONE,
                       (gobject.TYPE_PYOBJECT,))
        
gobject.signal_new(_tv_selected_signal, TreeviewAction,
                       gobject.SIGNAL_RUN_LAST,
                       gobject.TYPE_NONE,
                       (gobject.TYPE_PYOBJECT,))


class ScaleAction(gtk.Action) :
       
    def __init__(self, name, label = None, tooltip = None, stock_id = None ) :
        gtk.Action.__init__(self, name, label, tooltip, stock_id)
        self._proxies = [] 
    
    def connect_proxy(self, proxy) :
        proxy.connect('scale-changed', self._value_changed_cb)
        self._proxies.append(proxy)
    
    def get_proxies(self) :
        return self._proxies
  
    def _value_changed_cb(self, widget, value) :
        
        self.emit('scale-changed', value)
    


gobject.signal_new('scale-changed', ScaleAction,
                       gobject.SIGNAL_RUN_LAST,
                       gobject.TYPE_NONE,
                       (gobject.TYPE_PYOBJECT,))


class AppActions(Signallable, Loggable):
    __signals__ = {
            'open-project' : ["path"],
            'new-project' : [],
            'quit-app' : [],
            'show-prefs' : [],
            'show-about' : [],  

            'save-project' : [],
            'save-as-project' : ["path"],
            'close-project' : [],
            'acquisition-mode' : [],
            'timeline-mode' : [],
            'image-view' : ["origin", "image_name", "position"],
            'import-sound' :  ["path"],
            'import-images' : ["paths"],

            'start-acquistion' : [],
            'stop-acquistion' : [],
            'take-snapshot' : [],
            'active-mixer' : ['is_active'],

            'change-timeline-order' : ['objs', 'target'],
            'insert-image-on-timeline' : ['objs', 'target'],
            'remove-image-on-timeline' : ['objs'],
            'active-sound' :[],
            'deactive-sound' :[],
            'sound-stop-with-video': ['is_stop'],
            'change-framerate': ['fpi'],



            }
    
    
    _MODE_ACQ = 'AcquisitionMode'
    _MODE_TIMELINE = 'TimelineMode'
    
    

    def __init__(self, gui, app):
        
        Signallable.__init__(self)
        Loggable.__init__(self)
        self.gui = gui
        self.app = app
        self.builder_actions = self.gui.builder.ui_actions  
        self._action_groups = {}
        self._init_actions()
        self._create_actions()
        self._connect_to_app(self.app)
    
    #
    # public methods
    #
    def get_action(self, action_name) :
        _action = None
        for _action_group in self._action_groups.values() :
            _action = _action_group.get_action(action_name)
            if _action :
                break
        if not _action:
            self.error('No action found for action name :%s ', action_name)
        return _action


    def _init_actions(self) :
        #
        # GLOBAL APPLICATION action
        #

        # simple actions
        self._APP_ACTIONS = [
            (self.builder_actions[GUI_CONST.ACTION_OPEN], 
                self._open_cb),
            (self.builder_actions[GUI_CONST.ACTION_NEW],
                self._new_cb),
            (self.builder_actions[GUI_CONST.ACTION_QUIT],
                self._quit_cb),
            (self.builder_actions[GUI_CONST.ACTION_PREFERENCES],
                self._preferences_cb),
            (self.builder_actions[GUI_CONST.ACTION_ABOUT],
                self._about_cb),
            ]

        #
        # ACTIONS when project is loaded common for all modes 
        #

        # simple actions
        self._PROJECT_ACTIONS = [
            (self.builder_actions[GUI_CONST.ACTION_SAVE],
                self._save_cb),
            (self.builder_actions[GUI_CONST.ACTION_SAVE_AS],
                self._save_as_cb),
            (self.builder_actions[GUI_CONST.ACTION_CLOSE],
                self._close_cb),
            (self.builder_actions[GUI_CONST.ACTION_IMPORT_IMAGE],
                self._import_image_cb),
            (self.builder_actions[GUI_CONST.ACTION_EXPORT_VIDEO],
                self._export_video_cb),
            (self.builder_actions[GUI_CONST.ACTION_EXPORT_TOOL],
                self._export_tool_cb),
                                     ]
        

        # radio actions
        self._MODES_SIGNALS = {
            self._MODE_ACQ : 'acquisition-mode',
            self._MODE_TIMELINE : 'timeline-mode', }
        self._MODES_RADIO_ACTION = [
            (self._MODE_ACQ, gtk.STOCK_MEDIA_RECORD,
                '_Acqusition', '<Control>a', 'Acquisition Mode', 0),
            (self._MODE_TIMELINE, gtk.STOCK_EXECUTE,
                '_Timeline', '<Control>t', 'Timeline Mode', 1)
                    ]
                
        # treeview actions
        self._PROJECT_TV_ACTIONS = [
            (TreeviewAction(GUI_CONST.ACTION_TV_CAPTURE),
                self._tv_capture_change, self._tv_capture_selected ),
            (TreeviewAction(GUI_CONST.ACTION_TV_CHRONO),
                self._tv_chrono_change, self._tv_chrono_selected ),
            ]

       
        #
        # ACQUISITON mode actions
        #
        # simple actions
        
        # associate action icon
        self.builder_actions[GUI_CONST.ACTION_SNAPSHOT].set_stock_id(GUI_CONST.STOCK_SNAPSHOT)
        self._ACQ_ACTIONS = [
            (self.builder_actions[GUI_CONST.ACTION_SNAPSHOT],
                self._snapshot_cb)
            ]

        # toggle actions       
        self._ACQ_TOGGLE_ACTIONS = [ 
            (GUI_CONST.ACTION_ACQUISITION, GUI_CONST.STOCK_ACQUISITION,
                '_Acqusition', '<Control>m',
                'Start Stop acquisition', self._acquistion_cb),
            (GUI_CONST.ACTION_MIXER, None, 
                '_Mixer', None, 
                'Active/deactive mixer', self._mixer_cb)
            ]
        
        #
        # TIMELINE mode actions
        #
        
        # simple actions
        self._TIMELINE_ACTIONS = [
           (self.builder_actions[GUI_CONST.ACTION_IMPORT_SOUND],
                self._import_sound_cb),
            (self.builder_actions[GUI_CONST.ACTION_UP],
                self._move_up_cb),
            (self.builder_actions[GUI_CONST.ACTION_DOWN],
                self._move_down_cb),
            (self.builder_actions[GUI_CONST.ACTION_TRASH_CHRONO],
                self._delete_chrono_cb),
            (self.builder_actions[GUI_CONST.ACTION_MOVE_TO_CHRONO],
                self._move_to_chrono_cb),


            ]
        
        # toggle actions  
        self._TIMELINE_TOGGLE_ACTIONS = [ 
            (GUI_CONST.ACTION_SOUND_ACTIVE, None,
                '_Active Sound', None,
                'Active Deactive Sound', self._active_sound_cb),
            (GUI_CONST.ACTION_SOUND_STOP_WITH_VIDEO, None,
                'Sound stop with video', None,
                'Stop sound with video', self._sound_stop_with_video_cb),
            ]

        # Scale actions
        self._TIMELINE_SCALE_ACTIONS = [
            (ScaleAction(GUI_CONST.ACTION_FPI),
                self._fpi_value_changed_cb ),
            ]

           

    def _create_actions(self) :
        
        self._action_groups['GLOBAL'] = self._create_actions_app()

        self._action_groups['PROJECT'] = self._create_actions_project()
        self._action_groups['ACQUISITION'] = self._create_actions_acquistion()
        self._action_groups['TIMELINE'] = self._create_actions_timeline()
        self._create_tool_menu(self.gui.builder.ui_objects['toolbar1'])
        
        # TODO : EXPERIENCE
        """
        _action = self.get_action(GUI_CONST.ACTION_MOVE_TO_CHRONO)
        
        for _wdg in _action.get_proxies() :
            print 'Widget %s ,  prop gtk-button-image = %s'%(
                _wdg,
                _wdg.get_settings().props.gtk_button_images)
            print _action.get_stock_id() 
            print "use stock ",_wdg.get_use_stock()
            _wdg.set_use_stock(True)
            print "use stock ",_wdg.get_use_stock()
            print "image ",_wdg.props.image
            _img =  gtk.image_new_from_stock(_action.get_stock_id(), gtk.ICON_SIZE_BUTTON)
            _wdg.set_image(_img)
            print "image ",_wdg.props.image
            #_wdg.show_all()
        """

    def _create_actions_app(self) :
        _action_group = gtk.ActionGroup('ActionsApp')
        _action_group.set_sensitive(True)
        self._add_actions( self._APP_ACTIONS,  _action_group) 
        # connect quit
        _main_window = self.gui.builder.window
        _main_window.connect('destroy',
                self._destroy_cb,
                _action_group.get_action(GUI_CONST.ACTION_QUIT) )
        _main_window.connect('delete-event',
                self._destroy_cb,
                _action_group.get_action(GUI_CONST.ACTION_QUIT) )
        return _action_group

    def _create_actions_project(self) :
        _action_group = gtk.ActionGroup('ActionsProject')
                
        _action_group.add_radio_actions(self._MODES_RADIO_ACTION, 
                                       0, self._project_mode_cb)
        
       

        self._add_actions(self._PROJECT_ACTIONS, _action_group)

        self._add_treeview_actions(self._PROJECT_TV_ACTIONS,\
                _action_group)

        _action_group.set_sensitive(False)
        return _action_group

    def _create_actions_acquistion(self) :
        _action_group = \
                gtk.ActionGroup('ActionsAcquisition')
        
        _action_group.\
                add_toggle_actions(self._ACQ_TOGGLE_ACTIONS)
        self._add_actions(self._ACQ_ACTIONS, _action_group) 
        _action_group.set_sensitive(False)
        return _action_group

    def _create_actions_timeline(self) :
        _action_group = \
                gtk.ActionGroup('ActionsTimeline')
        
        self._add_actions(self._TIMELINE_ACTIONS, _action_group) 
        
        _action_group.\
                add_toggle_actions(self._TIMELINE_TOGGLE_ACTIONS)
        
        self._add_scale_actions(self._TIMELINE_SCALE_ACTIONS,\
                _action_group)

        _action_group.set_sensitive(False)
        return _action_group

    
    def _create_tool_menu(self, toolbar) :
        _action = self.get_action(GUI_CONST.ACTION_NEW)
        _toolitem = _action.create_tool_item()
        toolbar.insert(_toolitem, -1)

        _action = self.get_action(GUI_CONST.ACTION_OPEN)
        _toolitem = _action.create_tool_item()
        toolbar.insert(_toolitem, -1)
        
        _action = self.get_action(GUI_CONST.ACTION_SAVE)
        _toolitem = _action.create_tool_item()
        toolbar.insert(_toolitem, -1)


        _separator = gtk.SeparatorToolItem()
        toolbar.insert(_separator, -1)
        
        _action = self.get_action('AcquisitionMode')
        _toolitem = _action.create_tool_item()
        toolbar.insert(_toolitem, -1)
        
        _action = self.get_action('TimelineMode')
        _toolitem = _action.create_tool_item()
        toolbar.insert(_toolitem, -1)

        _separator = gtk.SeparatorToolItem()
        toolbar.insert(_separator, -1)
        
        _action = self.get_action(GUI_CONST.ACTION_IMPORT_IMAGE)
        _toolitem = _action.create_tool_item()
        toolbar.insert(_toolitem, -1)
        
        _action = self.get_action(GUI_CONST.ACTION_IMPORT_SOUND)
        _toolitem = _action.create_tool_item()
        toolbar.insert(_toolitem, -1)


    def _add_actions(self, actions, action_group) :
        for _action, _callback in actions :
            self.debug('connect %s', _action.get_name())
            _action.connect('activate', _callback)
            action_group.add_action(_action)
    
    def _add_treeview_actions(self, actions, action_group) :
        for _action, _cb_change, _cb_selected   in actions :
            self.debug('connect %s', _action.get_name())
            _action.connect('treeview-change', _cb_change)
            _action.connect('treeview-selected', _cb_selected)
            action_group.add_action(_action)
    
    def _add_scale_actions(self, actions, action_group) :
        for _action, _cb_value_changed in actions :
            self.debug('connect %s', _action.get_name())
            _action.connect('scale-changed', _cb_value_changed)
            action_group.add_action(_action)

    #
    #
    # actions callbacks
    #
    #
    
    #
    # global actions callbacks
    #
    def _destroy_cb(self, widget, unused, action) :
        action.activate()
        return True
    
    def _open_cb(self, action):
        _path = None
        # launch open project window
        _path = self.gui.windows.open_project()
        if _path :
            self.debug('emit gui open-project signal : %s', _path)
            self.emit('open-project', _path)
    
    def _new_cb(self, action) :
        self.debug('emit gui new-project signal ')
        self.emit('new-project' )
    
    def _quit_cb(self, action ) :
        self.debug('emit signal quit')
        self.emit('quit-app')
        return True
    
    def  _preferences_cb(self, action ) :
        # TODO : implement preferences dialog
        pass

    def _about_cb(self, action ) :
        # TODO : implement About dialog
        pass

   
    
    #
    # project common callbacks
    #
    def _project_mode_cb(self, action, current ) :
        self.debug("action project Mode - %s %s ",
                action.get_current_value(),
                current.get_name() )
        _signal =  self._MODES_SIGNALS[current.get_name()]
        self.debug("Emit %s ", _signal)
        self.emit(_signal)
    
    def _save_cb(self, action) :
        self.debug('emit save-project')
        self.emit('save-project')

    def _save_as_cb(self, action) :
        # launch dir chooser window
        _path = self.gui.windows.dir_chooser()
        
        if _path :
            self.debug('emit save-as-project : %s', _path)
            self.emit('save-as-project', _path)

    def _import_image_cb(self, action) :
        _paths = None
        _paths = self.gui.windows.import_dialog()
        if _paths :
            self.debug('emit import-images signal : %s', _paths)
            self.emit('import-images', _paths)


    def _export_video_cb(self, action) :
        # TODO implement
        pass

    def _export_tool_cb(self, action) :
        # TODO implement
        pass
     
    def _move_to_chrono_cb(self, action) :
        """ move from capture to chrono.
            Get traeeviews from action. transmit it to app
        """
        _action = self.get_action(GUI_CONST.ACTION_TV_CAPTURE)
        _tv_capture = _action.get_proxies()[0]
        
        _action = self.get_action(GUI_CONST.ACTION_TV_CHRONO)
        _tv_chrono = _action.get_proxies()[0]
        
        _images_name = _tv_capture.images_to_move()
        _tv_chrono.append_images(_images_name)
        

    def _close_cb(self, action) :
        self.debug('emit gui close-project signal')
        # TODO : ask save before
        self.emit('close-project')

    def _tv_capture_change(self, action, data) :
        self.debug('treeview capture change : %s', data)
             
    def _tv_capture_selected(self, action, data) :
        self.debug('emit image-view: %s', data)
        
        self.emit('image-view',data['origin'], data['image'].name, 0  ) 

    def _tv_chrono_change(self, action, data) :
        self.debug(' treeview chrono change : %s', data)
        data = data['chrono_images']
        if data.has_key('objs_to_move') :
            self.debug(
            'emit change-timeline-order - objs to move : %s, target : %s',
                    data['objs_to_move'],
                    data['row_target'])
            
            self.emit('change-timeline-order', 
                    data['objs_to_move'],
                     data['row_target'])
        
        elif data.has_key('objs_to_insert') :
            self.debug(
            'emit insert-image-on-timeline - objs to insert : %s, target :%s',
                    data['objs_to_insert'],
                    data['row_target'])
            self.emit('insert-image-on-timeline', 
                    data['objs_to_insert'],
                     data['row_target'])

        elif data.has_key('objs_to remove'):
            self.debug('emit remove-image-on-timeline - objs to remove : %s',
                    data['objs_to remove'])
            
            self.emit('remove-image-on-timeline', 
                    data['objs_to remove'])

    def _tv_chrono_selected(self, action, data) :
        self.debug(' emit image-view : %s', data)
        self.emit('image-view',data['origin'], data['image'], data['position'] ) 





    #
    # acquistion callbacks
    #
    def _acquistion_cb(self, action) :
        self.debug("action Acquisition callback")
        if action.get_active() == True :
            self.debug('emit start-acquistion')
            self.emit('start-acquistion')
        else :
            self.debug('emit stop-acquistion')
            self.emit('stop-acquistion')
    
    def _snapshot_cb(self, action) :
        self.debug("action SnapShot callback")
        self.emit('take-snapshot')
     
    def  _mixer_cb(self, action) :
        
        self.debug('emit active-mixer :%s', action.get_active())
        self.emit('active-mixer',action.get_active() ) 


    #
    # timeline callbacks
    #

    def _import_sound_cb(self, action) :
        dialog = gtk.FileChooserDialog("Import sound",
                               None,
                               gtk.FILE_CHOOSER_ACTION_OPEN,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("Sounds")
        filter.add_mime_type("audio/ogg")
        #filter.add_pattern("*.ogg")
        dialog.add_filter(filter)
    
        response = dialog.run()
        path = None
        if response == gtk.RESPONSE_OK:
            path = dialog.get_filename()
        elif response == gtk.RESPONSE_CANCEL:
            print 'Closed, no files selected'

        dialog.destroy() 
        # TODO : for test purpose
        #path = '/home/nico/Documents/luciole/Track.ogg'
        
        if path :
            self.debug('emit import-sound - %s', path)
            self.emit('import-sound', path)
        
    def _active_sound_cb(self, action) :
        
        
        if action.get_active() == True :
            self.debug('emit active-sound')
            self.emit('active-sound')
        else :
            self.debug('emit deactive-sound')
            self.emit('deactive-sound')
    
    def _sound_stop_with_video_cb(self, action) :
        self.debug('emit sound-stop-with-video :%s',action.get_active() )
        self.emit('sound-stop-with-video', action.get_active() )
    
    def _fpi_value_changed_cb(self, action, value) :
        self.debug('fpi value changed : %s frames per image', value)
        self.emit('change-framerate', value)
    
    
    def _move_up_cb(self, action) :
        _action = self.get_action(GUI_CONST.ACTION_TV_CHRONO)
        _proxy = _action.get_proxies()[0]
        self.debug ('move up : proxy %s', _proxy)
        _proxy.move_up()

    def _move_down_cb(self, action) :
        _action = self.get_action(GUI_CONST.ACTION_TV_CHRONO)
        _proxy = _action.get_proxies()[0]
        self.debug ('move down : proxy %s', _proxy)
        _proxy.move_down()
    
    def _delete_chrono_cb(self, action) :
        _action = self.get_action(GUI_CONST.ACTION_TV_CHRONO)
        _proxy = _action.get_proxies()[0]
        self.debug ('delete in chrono : proxy %s', _proxy)
        _proxy.remove()

    #
    #
    # methods used to communicate with app
    # 
    #

    def _connect_to_app(self, app) : 
        app.project_ctrl.connect('project-loaded', self._project_loaded_cb)
        app.project_ctrl.connect('project-closed', self._project_closed_cb)
        app.project_ctrl.connect('project-mode-changed', 
                self._project_mode_changed_cb)
        
        app.project_ctrl.ctrl_tmln.connect('sound-props-changed',
                self._sound_props_changed_cb)
        

        app.project_ctrl.ctrl_acq.connect('acquisition-started',
                self._acquisition_started_cb)

        app.project_ctrl.ctrl_acq.connect('acquisition-stopped',
                self._acquisition_stopped_cb)



    def _project_loaded_cb(self, ctrl, project ):
        """ project loaded cb """
        self._action_groups['PROJECT'].set_sensitive(True)
        
        # upate GUI with project options
        _fpi =  int(project.props['fpi'])
        _action = self.get_action(GUI_CONST.ACTION_FPI)
        _proxy = _action.get_proxies()[0]
        self.debug(' proxy %s', _proxy)
        _index =1        
        # loop conversion fps table, the hscale need the key value
        for (_key, _value) in LCONST.VIDEO_FPS_TABLE.iteritems() :
            #  find the position 
            (_v1, _v2) = _value 
            if _fpi == _v2 : 
                _index = _key
        _proxy.set_value(_index)

    def _project_closed_cb(self, ctrl):
        """ project closed cb """
        self._action_groups['PROJECT'].set_sensitive(False)
        # TODO : intermediate solution set active on project load
        #self._action_groups['ACQUISITION'].set_sensitive(False)
        #self._action_groups['TIMELINE'].set_sensitive(False)

    def _project_mode_changed_cb(self, ctrl, mode ):
        if mode == 'TIMELINE' :
            self._action_groups['TIMELINE'].set_sensitive(True)
            self._action_groups['ACQUISITION'].set_sensitive(False)
        elif mode == "ACQUISITION" :
            self._action_groups['TIMELINE'].set_sensitive(False)
            self._action_groups['ACQUISITION'].set_sensitive(True)




    def _sound_props_changed_cb(self, 
            ctrl_tmln, active = None , stop_with_video = None) :
        """ sound-props-changed cb """
        if active is not None :

            _action = self.get_action(GUI_CONST.ACTION_SOUND_ACTIVE)
            self.debug('Active : action status before : %s, active request : %s', _action.get_active(), active ) 
            if active != _action.get_active() :
                
                # TODO : temp solution
                # in that case proxy and action are nont synced
                # ths display is correct, not the action active prop
                # who did not change
                # but the 'activate' signal is not emited
                for _proxy in _action.get_proxies() :
                    _action.block_activate_from(_proxy)
                    _proxy.set_active(active)
                    _action.unblock_activate_from(_proxy)
                self.debug("Active:  action status changed %s" % _action.get_active())
    
        if stop_with_video is not None :

            _action = self.get_action(GUI_CONST.ACTION_SOUND_STOP_WITH_VIDEO)
            self.debug(
                'Stop with video : action status before: %s, Stop with video request : %s',
                    _action.get_active(), 
                    stop_with_video )
            if stop_with_video != _action.get_active() :
                
                # TODO : temp solution
                # in that case proxy and action are nont synced
                # ths display is correct, not the action active prop
                # who did not change
                # but the 'activate' signal is not emited
                for _proxy in _action.get_proxies() :
                    _action.block_activate_from(_proxy)
                    _proxy.set_active(stop_with_video)
                    _action.unblock_activate_from(_proxy)

            if stop_with_video != _action.get_active() :
                #_action.set_active(stop_with_video)
                self.debug("Stop with video : action status changed %s" , 
                        _action.get_active())


    def _acquisition_started_cb(self, ctrl_acq, with_mixer):
        self.debug('RX acquisition-started : with_mixer : %s')
        _action = self.get_action(GUI_CONST.ACTION_ACQUISITION)
        if _action.get_active() != True :
            _action.set_active(True)
        
    def _acquisition_stopped_cb(self, ctrl_acq):
        self.debug('RX acquisition-stopped ')
        _action = self.get_action(GUI_CONST.ACTION_ACQUISITION)
        if _action.get_active() != False :
            _action.set_active(False)
