#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
project_modes.py
"""

# standard library imports
from gettext import gettext as _
import platform

# related third party imports
import gtk
from pitivi.log.loggable import Loggable
from pitivi.signalinterface import Signallable
from pitivi.signalgroup import SignalGroup
from pitivi.ui.viewer import PitiviViewer


# local application/library specific imports
import luciole.gui.base.builder as GUI_BUILDER
import luciole.gui.constants as GUI_CONST
import luciole.constants as LCONST

from luciole.gui.viewer_widget import ModeViewer



class ModeWidget(Loggable, Signallable) :
    
    def __init__(self, 
                actions, 
                glade_file, 
                window = LCONST.MODE_MAIN_WINDOW, 
                ):
        Loggable.__init__(self)
        Signallable.__init__(self)
        self.actions = actions

        self.builder = GUI_BUILDER.UiBuilder(
                glade_file,
                window)
        # hide by default 
        self.builder.window.hide()
        
        self.widget = self.builder.ui_objects[GUI_CONST.MODE_MAIN_WIDGET]
        self.debug('widget : %s',  self.widget)
        self.widget.unparent()
        # remove link with gtk.window
        #_child = self.builder.window.get_child()
        #self.builder.window.remove(_child)
        
        
        self.connect_actions()
        # Connect to project.  We must remove and reset the callbacks when
        # changing project.
        self.project_signals = SignalGroup()   


    def connect_action(self, action, button) :
        _btn = self.builder.ui_objects[button]
        _action = self.actions[action]
        _action.connect_proxy(_btn)


    def connect_actions(self) :
        pass
    
    def _connect_to_ctrl(self) :
        pass

class TimelineWdg(PitiviViewer):

    def __init__(self) :
        PitiviViewer.__init__(self) 
         
           


class NoProjectWidget(ModeWidget) :
    
    def __init__(self, actions, app) :
        ModeWidget.__init__(self, actions, 
                LCONST.NO_PROJECT_MODE_GLADE_PATH,
                )
    
    def connect_actions(self) :
        # connect open button
        ModeWidget.connect_action(self,
                GUI_CONST.ACTION_OPEN, 
                GUI_CONST.NO_PROJECT_OPEN_BTN)
        
        #connect new button
        ModeWidget.connect_action(self,
                GUI_CONST.ACTION_NEW,
                GUI_CONST.NO_PROJECT_NEW_BTN)


class ViewerWidget(gtk.DrawingArea, Loggable):
    """
    Widget for displaying properly GStreamer video sink
    """

    __gsignals__ = {}

    def __init__(self, action = None):
        gtk.DrawingArea.__init__(self)
        Loggable.__init__(self)
        self.set_size_request(360,288)
        self.action = action # FIXME : Check if it's a view action
        self.unset_flags(gtk.SENSITIVE)
        for state in range(gtk.STATE_INSENSITIVE + 1):
            self.modify_bg(state, self.style.black)
        

    def do_realize(self):
        gtk.DrawingArea.do_realize(self)
        if platform.system() == 'Windows':
            self.window_xid = self.window.handle
        else:
            self.window_xid  = self.window.xid

        self.debug("window xid = %s, state %s",   self.window_xid, self.state )

    def set_bg_white(self):
        for state in range(gtk.STATE_INSENSITIVE + 1):
            self.modify_bg(state, self.style.white)
    
    def set_bg_black(self):
        for state in range(gtk.STATE_INSENSITIVE + 1):
            self.modify_bg(state, self.style.black)

class AcquisitionWidget(ModeWidget) :
    
    MODES = (   "INVALID",
                "NO_ACQUISITION",
                "ACQUISITION",
                "ACQUISITION_WITH_MIXER",
                )

    
    
    def __init__(self, actions, app) :
        ModeWidget.__init__(self, 
                actions,
                LCONST.ACQ_MODE_GLADE_PATH, 
                )
                
        self.app = app
        
        self._mode_call = {
            'INVALID' : \
                self._display_no_acq,
            'NO_ACQUISITION' : \
                self._display_no_acq,
            'ACQUISITION' : \
                self._display_acq,
            'ACQUISITION_WITH_MIXER' : \
                self._display_acq_with_mixer,
                }
        
        self._mode = "INVALID"
        self._mode_call[self._mode]()
        self.debug('Init done')

        #self._connect_to_ctrl_acq(self.app.project_ctrl.acq_ctrl)

    def connect_actions(self) :
        
        # connect start stop acquistion
        ModeWidget.connect_action(self,
                GUI_CONST.ACTION_ACQUISITION, 
                GUI_CONST.ACQUISITION_BUTTON)
        
        # connect snapshot
        ModeWidget.connect_action(self,
                GUI_CONST.ACTION_SNAPSHOT, 
                GUI_CONST.SNAPSHOT_BUTTON)
        
        # get mixer check_button 
        ModeWidget.connect_action(self,
                GUI_CONST.ACTION_MIXER, 
                GUI_CONST.MIXER_CHECK_BUTTON)

           
    def connect_to_project(self, project):
        """Connect signal handlers to a project.

        This first disconnects any handlers connected to an old project.
        If project is None, this just disconnects any connected handlers.

        """
        self.project_signals.connect(
            project, "acquisition-started", None, self.acq_started_cb)
        self.project_signals.connect(
            project, "acquisition-stopped", None, self.acq_stopped_cb)
        self.project_signals.connect(
            project, "snapshot-on-progress", None, self.snapshot_on_progress_cb)
        self.project_signals.connect(
            project, "snapshot-taken", None, self.snapshot_taken_cb)


    def _disconnect_from_ctrl_acq(self, ctrl_acq) :
        ctrl_acq.connect.disconnect_by_functions(self.acq_started_cb)
        ctrl_acq.connect.disconnect_by_functions(self.acq_stopped_cb)
        ctrl_acq.connect.disconnect_by_functions(self.snapshot_on_progress_cb)
        ctrl_acq.connect.disconnect_by_functions(self.snapshot_taken_cb)

    def acq_started_cb(self, ctrl_acq, with_mixer = False) :
        if with_mixer :
            self._mode = "ACQUISITION_WITH_MIXER"
            self._mode_call[self._mode]()

        else :
            self._mode = "ACQUISITION"
            self._mode_call[self._mode]()

    def acq_stopped_cb(self, ctrl_acq) :
        self.debug('RX signal acquisition-stopped ')
        self._mode = "NO_ACQUISITION"
        self._mode_call[self._mode]()
    
    def snapshot_on_progress_cb(self, ctrl_acq) :
        # set button snapshot insensitive
        self.debug('RX signal snapshot-on-progress')
        _btn = self.builder.ui_objects[GUI_CONST.SNAPSHOT_BUTTON]
        _btn.set_sensitive(False)

    def snapshot_taken_cb(self, ctrl_acq, rush_image) :
        self.debug('RX signal snapshot-taken')
        # set button snapshot insensitive
        _btn = self.builder.ui_objects[GUI_CONST.SNAPSHOT_BUTTON]
        _btn.set_sensitive(True)


    
    def _display_no_acq(self) :
        # hide snapshot and mixer
        self.builder.ui_objects[GUI_CONST.ACQUISITION_CAPTURE_WDG].hide()
    
    def _display_acq(self):
        self.builder.ui_objects[GUI_CONST.ACQUISITION_CAPTURE_ALPHA_WDG].hide()
        self.builder.ui_objects[GUI_CONST.ACQUISITION_CAPTURE_WDG].show()

    def _display_acq_with_mixer(self):
        self.builder.ui_objects[GUI_CONST.ACQUISITION_CAPTURE_ALPHA_WDG].show()
        self.builder.ui_objects[GUI_CONST.ACQUISITION_CAPTURE_WDG].show()




class ComboModeWidget(object) :

    def __init__(self, modes, changed_cb ) :
        
        combobox = gtk.combo_box_new_text()
        
        combobox.append_text('Select a mode:')
        for mode in modes : 
            combobox.append_text(mode)
        combobox.connect('changed', changed_cb)
        combobox.set_active(0)
        self.combobox = combobox 




class ProjectModeWidget(object) :

    MODES = ['NO_PROJECT', 'ACQUISITION', 'TIMELINE']    
    
    def _get_mode(self) :
        """ mode getter """
        return self._mode
    def _set_mode(self, mode) :
        """ mode setter """
        if mode  in self.MODES and mode != self._mode :
            self._mode_call[mode]()
    mode = property(_get_mode,
                    _set_mode,
                    None,
                    """ mode widget """)

    

    def __init__(self, parent= None, actions=None, app = None ) :
        self.parent = parent
        self.actions = actions
        self.app = app
        self.combomode = ComboModeWidget(self.MODES, self.changed_cb)
        self._mode = None
        self.active_widget = None
        
        self._mode_call = {
            'NO_PROJECT' : \
                self._no_project_mode,
            'ACQUISITION' : \
                self._acqusition_mode,
            'TIMELINE' : \
                self._timeline_mode,
                }
        self.widgets = {}
        self.klasses = {}

        self.viewer_widget = None 
        self._init_widgets(self.actions, self.app )
        # by default : NO_PROJECT mode
        self._no_project_mode()

# TODO : project mode to remove or change goal

    def _init_widgets(self, actions,  app) :
        self.viewer = ModeViewer(actions)
        self.parent.add(self.viewer)
        # TODO temp for test only ;
        for _mode in self.MODES  :
            self.widgets[_mode] = self.viewer
            self.klasses[_mode] = self.viewer

    def _init_viewer(self, vbox) :
        self.viewer_widget =  ViewerWidget()
        aspect_frame = gtk.AspectFrame("Viewer",0.1,0.1,1.25,False)
        aspect_frame.set_size_request(360,288)
        aspect_frame.add(self.viewer_widget)
        aspect_frame.show()
        vbox.pack_start(aspect_frame, fill = True, expand = True, padding =0 )
        
    def _init_acq_widget(self, actions, app ) :
        _klass = AcquisitionWidget(actions, app)
        self.klasses['ACQUISITION'] = _klass
        self.widgets['ACQUISITION'] = _klass.widget

    def _init_no_project_widget(self, app) :
        _klass = NoProjectWidget(self.actions, app)
        self.klasses['NO_PROJECT'] = _klass
        self.widgets['NO_PROJECT'] = _klass.widget
                    
            
    def _init_timeline_widget(self, app) :
        _klass = TimelineWdg() 
        self.klasses['TIMELINE'] = _klass
        self.widgets['TIMELINE'] =  _klass                                     

    def changed_cb(self, combobox) :
        model = combobox.get_model()
        index = combobox.get_active()
        if index:
            mode = model[index][0]
            print 'I like', mode, 'mode !'
            self._mode_call[mode]()
  
    def _no_project_mode(self) :
        self._mode = 'NO_PROJECT'
        #self._set_active_widget('NO_PROJECT')
    
    def _acqusition_mode(self) :
        self._mode = 'ACQUISITION'
        #self._set_active_widget('ACQUISITION')

    def _timeline_mode(self) :
        self._mode = 'TIMELINE' 
        #self._set_active_widget('TIMELINE')

    def _set_active_widget(self, mode) :
        
        if self.active_widget != None :
            # remove widget from view
            print "remove : ", gtk.Buildable.get_name(self.active_widget)
            self._mode_aspectf.remove(self.active_widget)
        self._mode = mode
        self.active_widget = self.widgets[self._mode]
        self.active_widget.show()
        self._mode_aspectf.add(self.active_widget)

    def update_framerate(self, fpi) :
        self.fpi = fpi
        if self._mode == "TIMELINE"  :
            # todo update on timeline
            pass

    def connect_to_ctrl(self, mode, ctrl) :
        pass
        """
        if mode in self.MODES :
            self.klasses[mode].connect_to_project(ctrl)
        else :
            print 'UNKNOW mode : %s'%mode
        """
        


def main() :
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.connect('destroy', lambda w: gtk.main_quit())
    
    
    
    aspect_frame = gtk.AspectFrame("mode", # label
                                       0.5, # center x
                                       0.5, # center y
                                       #2, # xsize/ysize = 2
                                       ) # ignore child's aspect
    

    ModeWdg = ModeWidget(aspect_frame)
    
    vbox = gtk.VBox(False, 0)
    vbox.pack_start(ModeWdg.combomode.combobox)
    vbox.pack_start(aspect_frame)

    window.add(vbox)
    window.show_all()

    ModeWdg.mode = 'ACQUISITION'
    
    gtk.main()
    return
    
if __name__ == "__main__":
    main()



