#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#   
# 
#
"""
    Luciole Treeciew interface
"""


# standard library imports
# N/A

# related third party imports
import gtk
from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable
from pitivi.signalgroup import SignalGroup

# local application/library specific imports
import luciole.gui.treeviews.luciole_tree as LT
import luciole.constants as LCONST
import luciole.gui.constants as GUI_CONST
import luciole.base.exceptions as LEXCEP 


class CaptureTree(LT.LucioleTree) :
    """ Class for mamaging treeview on capture """
    # Allow only Drag  On app  
    targets_source = [
                ("text/x-luciole-images", gtk.TARGET_SAME_APP, 1)
                ]
    # Disallow all type of Drop in capture, the image sequence cannot 
    # be changed
    targets_dest = []
    

    def __init__(self, capture_list, images_source, gui_actions):
        _action = gui_actions.get_action(GUI_CONST.ACTION_TV_CAPTURE)

        LT.LucioleTree.__init__(self, 
                                capture_list,
                                images_source,
                                _action)
        self.type = LCONST.CAPTURE
        
        
        self.tvcolumn.set_property('title',"  Capture ")     
        self.set_name('luciole_capture_tree')
    
    def images_to_move(self) :
        """ 
        return list of images to move from capture to chrono
        """
        _model, _path_list = self._get_selected_rows()
        _images = None
        if _path_list :
            _images = [_model[_path][1] for _path in _path_list]
        return _images


class ChronoTree(LT.LucioleTree) :
    """ Class for mamaging treeview on chrono """
    # redefinition of targets to allow DnD only inside the treeview
    
    # Drag only on treeview 
    targets_source = [ ('tree_model', gtk.TARGET_SAME_WIDGET, 0),
                ]
    
    # Drop from treeview or montage treeview
    targets_dest = [ ('tree_model', gtk.TARGET_SAME_WIDGET, 0),
                ("text/x-luciole-images", gtk.TARGET_SAME_APP, 1),
               ]
                
                        
    def __init__(self, capture_list, images_source, gui_actions ):
        _action = gui_actions.get_action(GUI_CONST.ACTION_TV_CHRONO)
        LT.LucioleTree.__init__(self,
                                capture_list,
                                images_source, 
                                _action)
        self.type = LCONST.CHRONO
        self.tvcolumn.set_property('title',"  Montage ")
        
        # This signal is used to manage the selection of a row and display
        # image preview 
        self.get_selection().connect('changed', self._on_selection_changed)


   
   
    def move_up(self) :
        """ Move up one position a selection in treeview"""
        _model, _path_list = self._get_selected_rows()
        if _path_list :
            # Move one position : get the path of the previous row
            # in selection if exist
            _path = _path_list[0]
            if _path[0] > 0 :
                _path = (_path[0]-1,)
            else :
                _path = None
            # to indicate insetion before
            _position = gtk.TREE_VIEW_DROP_BEFORE

            self._move_rows(_path_list, _path, _position, _model)

    def move_down(self) :
        """ Move down one position a selection in treeview"""
        _model, _path_list = self._get_selected_rows()
        if _path_list :
            # Move one position : get the path of the last row 
            # in selection if exist
            _path = _path_list[-1]
            if _path[0] < ( len(_model) -1 )  :
                _path = (_path[0]+ 1,)
            else :
                _path = None
            # to indicate insetion after
            _position = gtk.TREE_VIEW_DROP_AFTER

            self._move_rows(_path_list, _path, _position, _model)
   
    def get_position_selected_row(self):
        """
        Get the position/index of a selected row.
        Only if one image is selected
        """
        _model, _path_list = self._get_selected_rows()
        _pos = 0
        if len(_path_list) == 1 :
            # only one row selected
            # return the index
            _pos = _path_list[0][0]
        return _pos

    def _on_selection_changed(self, selection) :
        """
        selction changed :  display image preview.  
        only if one image is selected  
        """
        (_model, _path_list) = selection.get_selected_rows()
        if len(_path_list) == 1 :
            _path = _path_list[-1]
            _image = self._images_source.get_image(self.model[_path][1])
            _data = {
                'origin': self.type,
                'image' : _image.name,
                'position' : _path[0],
                    }
            self.emit('treeview-selected', _data)

class Treeviews(Loggable, Signallable) :

    def __get_treeviews(self) :
        """ treeviews getter"""
        if self._treeviews != {} :
            return self._treeviews
        else :
            _err_msg = ' Treeviews is empty' 
            raise LEXCEP.LucioException, _err_msg
    treeviews =  property(  __get_treeviews,
                            None,
                            None,
                            """ treeviews structure """)
 
    def __init__(self, gui_objects, gui_actions, app) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        
        self.gui_objects = gui_objects
        self.gui_actions = gui_actions
        self._treeviews = {}
    
        # Store default treeviews : used for clear it
        self.__default_tv_capture = self.gui_objects[GUI_CONST.TV_CAPTURE]
        self.__default_tv_chrono = self.gui_objects[GUI_CONST.TV_CHRONO]
        
        
        # Connect to project.  We must remove and reset the callbacks when
        # changing project.
        self.project_signals = SignalGroup()   
        self.connect_to_app(app)


    def clear(self) :
        """ clear treeviews with default blank treeview (from glade) """
        #
        # clear capture_treeview
        #
        _tv = self.gui_objects[GUI_CONST.TV_CAPTURE]
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            # replace it by default one.
            _tv_parent.add(self.__default_tv_capture)
            self.gui_objects[GUI_CONST.TV_CAPTURE] = self.__default_tv_capture
        #
        # clear chrono_treeview
        #
        _tv = self.gui_objects[GUI_CONST.TV_CHRONO]
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            # replace it by default one.
            _tv_parent.add(self.__default_tv_chrono)
            self.gui_objects[GUI_CONST.TV_CHRONO] = self.__default_tv_chrono
    
        # clear treeviews strcture
        self._treeviews = {}
        


    def load(self, project) :
        """ This mrgof load the treeviews : capture and chrono 
         both treeview defined in the xml/glade file are replaced by
         the Capture_tree and Chrono_tree who are inherited from gtk.treeview
        """
        #
        # load capture treeview
        #
        _tv = self.gui_objects[GUI_CONST.TV_CAPTURE] 
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            
            # Create capture treeview
            _tv_new = CaptureTree(    
                    project.props['capture_images'], 
                    project.rush,
                    self.gui_actions
                    )
            # add it un gui
            _tv_parent.add(_tv_new)
            _tv_new.show()
            self.gui_objects[GUI_CONST.TV_CAPTURE] = _tv_new
            self._treeviews[LCONST.CAPTURE] = _tv_new
            # auto resize columns
            _tv_new.columns_autosize()
        else:
            lerr =  " unable to find treeview capture "
            raise LEXCEP.LucioException, lerr

        #
        # load chrono treeview
        #
        _tv = self.gui_objects[GUI_CONST.TV_CHRONO] 
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            
            # Create capture treeview
            _tv_new = ChronoTree(    
                    project.props['chrono_images'], 
                    project.rush,
                    self.gui_actions
                    )
            # add it un gui
            _tv_parent.add(_tv_new)
            _tv_new.show()
            self.gui_objects[GUI_CONST.TV_CHRONO] = _tv_new
            self._treeviews[LCONST.CHRONO] = _tv_new
            # auto resize columns
            #_tv_new.columns_autosize()
        else:
            lerr =  " unable to find treeview chrono "
            raise LEXCEP.LucioException, lerr
    
    def connect_to_app(self, app) : 
        app.project_ctrl.connect('project-loaded', self.project_loaded_cb)
        app.project_ctrl.connect('project-closed', self.project_closed_cb)


    def disconnect_from_app(self, app)  :
        app.project_ctrl.disconnect_by_function(self.project_loaded_cb)
        app.project_ctrl.disconnect_by_function(self.project_closed_cb)

    
    def project_loaded_cb(self, project_ctrl, project) :
        self.debug('RX signal project-loaded. project is %s',
                project.props['project_name'] ) 

        # load treeviews 
        self.load(project)

        # connect to project signals
        self._connectToProject(project_ctrl)
    
    def  project_closed_cb(self, project_ctrl) :
        self.debug('RX signal project-closed. project is %s',
                project_ctrl.project.props['project_name'] ) 
        
        # clear treeview
        self.clear()
        # disconnect snaphot taken signal
        project_ctrl.ctrl_acq.disconnect_by_function(self.snapshot_taken_cb)
        



    def snapshot_taken_cb(self, ctrl_acq, rush_image) :
        """ add snapshot to capture treeview """
        self.debug('RX signal snapshot-taken. image name : %s', 
                rush_image.name)
        self.debug('append snasphot %s to treeview', rush_image.name)
        self._treeviews[LCONST.CAPTURE].append_images([rush_image.name]) 
        

    def _connectToProject(self, project):
        """Connect signal handlers to a project.

        This first disconnects any handlers connected to an old project.
        If project is None, this just disconnects any connected handlers.

        """
        self.project_signals.connect(
            project.ctrl_acq, "snapshot-taken", None, self.snapshot_taken_cb)

