#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- mode: python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# copyright nicolas bertrand (nico@inattendu.org), 2009-2010
#
# this file is part of luciole.
#
#    luciole is free software: you can redistribute it and/or modify
#    it under the terms of the gnu general public license as published by
#    the free software foundation, either version 3 of the license, or
#    (at your option) any later version.
#
#    luciole is distributed in the hope that it will be useful,
#    but without any warranty; without even the implied warranty of
#    merchantability or fitness for a particular purpose.  see the
#    gnu general public license for more details.
#
#    you should have received a copy of the gnu general public license
#    along with luciole.  if not, see <http://www.gnu.org/licenses/>.
#
#
"""
webcam_properties_widget.py :
	Display webcam properties widget
"""
# standard library imports
# N/A

# related third party imports
import gtk
from pitivi.signalinterface import Signallable
from pitivi.log.loggable import Loggable

# local application/library specific imports

class WebcamField(Loggable, Signallable) :
    __signals__ = {
                'combo-changed' : ['field','value'],
        }

    def __init__(self, field_name, value, widget_type = 'combo' ) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        self.field_name = field_name
        self.value = value
        self.widget_type = widget_type
        self._combo_idx = []
        self._combo = None
        self._range = None
        self.widget = self._createUi()

    def _createUi(self) : 
        _bbox =  gtk.HBox()
        _boxalign = gtk.Alignment(xalign=0.0, yalign=0.0)
        _boxalign.add(_bbox)
        
        _label = gtk.Label(str(self.field_name))
        _bbox.pack_start(_label, expand=False)

        if isinstance(self.value, list) :
            if self.widget_type == 'combo' :
                self._combo = self._create_combo(_bbox)
            elif self.widget_type == 'range':
                self._range = self._create_range(_bbox)
            else :
                self.error('Unknow widget type %s', self.widget_type)
        else :
            _label = gtk.Label(str(self.value))
            _bbox.pack_start(_label, expand=False)
        return _boxalign
    
    def _create_combo(self, bbox) :
        _combo = gtk.combo_box_new_text()
        self.debug('create combobox for field %s', self.field_name)
        for _idx, _value in enumerate(self.value) :
            _combo.append_text(str(_value))
            self._combo_idx.append(str(_value))
        _combo.connect('changed', self.combobox_defintion_changed_cb)
        bbox.pack_start(_combo, expand=False)
        return _combo
    
    def _create_range(self, bbox) :
        
        _adj = gtk.Adjustment(
                float(self.value[0]),
                float(self.value[0]),
                float(self.value[1]),
                0.1,
                1.0,
                1.0)
        _scale = gtk.HScale(_adj)
        _scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
        _scale.set_digits(0)
        _scale.set_value_pos(gtk.POS_RIGHT)
        _scale.set_draw_value(True)
        _scale.set_size_request(500, 500)
        _scale.connect('value-changed', self.vscale_value_changed_cb)
        bbox.pack_start(_scale, expand=True, fill = True )
        return _scale

    def combobox_defintion_changed_cb(self, widget) :

        self.emit('combo-changed', self.field_name, widget.get_active_text() )
    
    def vscale_value_changed_cb(self, widget) :
        self.emit('combo-changed', self.field_name, widget.get_value() )


    def set_active(self, value) :
        if self._combo is not None :
            try :
                _idx = self._combo_idx.index(value)
            except :
                self.error('value %s not found in combo', value)
            else :
                self._combo.set_active(_idx)


class WebcamPropsWdg(Loggable, Signallable) :
    """ Webcam Properties """ 
   
    __signals__ = {
        'prop-changed' : ['prop','value'],
        'webcam-changed' : ['webcam'],
        }

    _MAIN_WDG = "vbox_main"
    

    def __init__(self, parent_wdg) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        
        self.prop_widgets = {}
        
        self._main_window = gtk.VBox()

        parent_wdg.add(self._main_window)
        
        self.prepare_ui()
        
        # Show the window and all its children
        self._main_window.show_all()

        

    def prepare_ui(self) :
        pass
           
    
    def get_active_value(self, combo_type) :
        if combo_type == 'definition' : 
            _combo = self.builder.get_object(self._COMBO_DEFINITIONS)
        else :
            self.error('unknown combo type %s',combo_type)
            return None
        _value = self._get_data_from_model(_combo)
        self.debug('active value : %s'%_value)
        return _value

    def set_definitions(self, definitions) :
        self.set_prop('definition', definitions) 
    
    def set_active_defintion(self, definition) :
        _wdg = self.prop_widgets['definition']
        _wdg.set_active(definition)


    def set_videotypes(self, videotypes) :
        self.set_prop('videotype', videotypes) 

    def set_active_videotype(self, value) :
        self.set_active_prop('videotype',value) 
    
    
    def set_prop(self, prop, prop_values, wdg_type='combo' ) :
        _wdg = WebcamField(prop, prop_values, wdg_type )
        if isinstance(prop_values, list) :
            _wdg.connect('combo-changed', self.combo_changed_cb)
            self.prop_widgets[prop] = _wdg
        self._main_window.pack_start(_wdg.widget) 
        _wdg.widget.show_all()

    
    def set_active_prop(self, prop, active_value):
        _wdg = self.prop_widgets[prop]
        _wdg.set_active(active_value)
    
    def remove_prop(self, prop) :
        if prop in self.prop_widgets :
        
            self.debug('remove prop %s', prop)
            _wdg =  self.prop_widgets[prop]
        
            self._main_window.remove(_wdg.widget)
            del self.prop_widgets[prop]
    
    def remove_all_props(self) :
        _props =  self.prop_widgets.keys()
        for _prop in _props:
            _wdg = self.prop_widgets.pop(_prop)
            self._main_window.remove(_wdg.widget)
            _wdg.disconnect_by_function(self.combo_changed_cb)
            self.debug("remove widget prop %s", _prop)


    def combo_changed_cb(self, widget, prop, value) :
        self.debug('emit prop-changed : %s=%s', prop, value)
        self.emit('prop-changed', prop, value)
    

    def display_webcams(self, _webcams) :
        _wdg = WebcamField('webcam-name', _webcams)
        if isinstance(_webcams, list) :
            _wdg.connect('combo-changed', self._webcam_changed_cb)
            self.webcam_name = _wdg
        """
        _keys = self.prop_widgets.keys()
        for _key in _keys :
            self.remove_prop(_key)
        """

        self._main_window.pack_start(_wdg.widget) 
        _wdg.widget.show_all()
    
    def _webcam_changed_cb(self, widget, prop, value) :
        self.debug('emit webcam-changed : %s=%s', prop, value)
        
        self.emit('webcam-changed', value)




