#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    webcam detetction widget
"""


# standard library imports
from gettext import gettext as _

# related third party imports

import gtk
import gobject


import gst


from pitivi.log.loggable import Loggable
from pitivi.signalinterface import Signallable

# for test
from pitivi.encode import RenderSinkFactory, RenderFactory
from pitivi.action import RenderAction
from pitivi.factories.base import SinkFactory
from pitivi.pipeline import Pipeline
from pitivi.settings import StreamEncodeSettings, RenderSettings

# local application/library specific imports
import luciole.media.webcam_detect.webcam_detection as WBCM_DTCT

# for test
from luciole.media.lgst.webcam_factory import WebcamSourceFactory

class WebcamField(Loggable, Signallable) :
    __signals__ = {
                'combo-changed' : ['field','value'],
        }

    def __init__(self, field_name, value, widget_type = 'combo' ) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        self.field_name = field_name
        self.value = value
        self.widget_type = widget_type
        self._combo_idx = []
        self._combo = None
        self._range = None
        self.widget = self._createUi()

    def _createUi(self) : 
        _bbox =  gtk.HBox()
        _boxalign = gtk.Alignment(xalign=0.0, yalign=0.0)
        _boxalign.add(_bbox)
        
        _label = gtk.Label(str(self.field_name))
        _bbox.pack_start(_label, expand=False)

        if isinstance(self.value, list) :
            if self.widget_type == 'combo' :
                self._combo = self._create_combo(_bbox)
            elif self.widget_type == 'range':
                self._range = self._create_range(_bbox)
            else :
                self.error('Unknow widget type %s', self.widget_type)
        else :
            _label = gtk.Label(str(self.value))
            _bbox.pack_start(_label, expand=False)
        return _boxalign
    
    def _create_combo(self, bbox) :
        _combo = gtk.combo_box_new_text()
        self.debug('create combobox for field %s', self.field_name)
        for _idx, _value in enumerate(self.value) :
            _combo.append_text(str(_value))
            self._combo_idx.append(str(_value))
        _combo.connect('changed', self.combobox_defintion_changed_cb)
        bbox.pack_start(_combo, expand=False)
        return _combo
    
    def _create_range(self, bbox) :
        
        _adj = gtk.Adjustment(
                float(self.value[0]),
                float(self.value[0]),
                float(self.value[1]),
                0.1,
                1.0,
                1.0)
        _scale = gtk.HScale(_adj)
        _scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
        _scale.set_digits(0)
        _scale.set_value_pos(gtk.POS_RIGHT)
        _scale.set_draw_value(True)
        _scale.set_size_request(500, 500)
        _scale.connect('value-changed', self.vscale_value_changed_cb)
        bbox.pack_start(_scale, expand=True, fill = True )
        return _scale

    def combobox_defintion_changed_cb(self, widget) :

        self.emit('combo-changed', self.field_name, widget.get_active_text() )
    
    def vscale_value_changed_cb(self, widget) :
        self.emit('combo-changed', self.field_name, widget.get_value() )


    def set_active(self, value) :
        if self._combo is not None :
            try :
                _idx = self._combo_idx.index(value)
            except :
                self.error('value %s not found in combo', value)
            else :
                self._combo.set_active(_idx)


class WebcamPropsWdg(Loggable, Signallable) :
    """ Webcam Properties """ 
   
    __signals__ = {
        'prop-changed' : ['prop','value'],
        'webcam-changed' : ['webcam'],
        }

    _MAIN_WDG = "vbox_main"
    

    def __init__(self, parent_wdg) :
        Signallable.__init__(self)
        Loggable.__init__(self)
        
        self.prop_widgets = {}
        self.webcam_name = None
        self._main_window = gtk.VBox()

        parent_wdg.add(self._main_window)
        
        self.prepare_ui()
        
        # Show the window and all its children
        self._main_window.show_all()

        

    def prepare_ui(self) :
        pass
           
  
    def set_definitions(self, definitions) :
        self.set_prop('definition', definitions) 
    
    def set_active_defintion(self, definition) :
        _wdg = self.prop_widgets['definition']
        _wdg.set_active(definition)


    def set_videotypes(self, videotypes) :
        self.set_prop('videotype', videotypes) 

    def set_active_videotype(self, value) :
        self.set_active_prop('videotype', value) 
    
    
    def set_prop(self, prop, prop_values, wdg_type='combo' ) :
        _wdg = WebcamField(prop, prop_values, wdg_type )
        if isinstance(prop_values, list) :
            _wdg.connect('combo-changed', self.combo_changed_cb)
            self.prop_widgets[prop] = _wdg
        self._main_window.pack_start(_wdg.widget) 
        _wdg.widget.show_all()

    
    def set_active_prop(self, prop, active_value):
        _wdg = self.prop_widgets[prop]
        _wdg.set_active(active_value)
    
    def remove_prop(self, prop) :
        if prop in self.prop_widgets :
        
            self.debug('remove prop %s', prop)
            _wdg =  self.prop_widgets[prop]
        
            self._main_window.remove(_wdg.widget)
            del self.prop_widgets[prop]
    
    def remove_all_props(self) :
        _props =  self.prop_widgets.keys()
        for _prop in _props:
            _wdg = self.prop_widgets.pop(_prop)
            self._main_window.remove(_wdg.widget)
            _wdg.disconnect_by_function(self.combo_changed_cb)
            self.debug("remove widget prop %s", _prop)


    def combo_changed_cb(self, widget, prop, value) :
        self.debug('emit prop-changed : %s=%s', prop, value)
        self.emit('prop-changed', prop, value)
    

    def display_webcams(self, _webcams) :
        _wdg = WebcamField('webcam-name', _webcams)
        if isinstance(_webcams, list) :
            _wdg.connect('combo-changed', self._webcam_changed_cb)
            self.webcam_name = _wdg

        self._main_window.pack_start(_wdg.widget) 
        _wdg.widget.show_all()
    
    def _webcam_changed_cb(self, widget, prop, value) :
        self.debug('emit webcam-changed : %s=%s', prop, value)
        
        self.emit('webcam-changed', value)


class FakeSinkFactory(SinkFactory):
    def __init__(self, factoryname="fakesink", *args, **kwargs):
        SinkFactory.__init__(self, *args, **kwargs)
        self.__factoryname=factoryname

    def _makeBin(self, output_stream=None):
        return gst.element_factory_make(self.__factoryname)

class  WebcamDetectionWidget(gtk.Frame, Loggable) :
    """ Assistant page for webcam detection """
    __gsignals__ = { 
           'webcams-detected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                               (gobject.TYPE_PYOBJECT,)),
           'webcam-not-detected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
               (gobject.TYPE_PYOBJECT,) ),
           'webcam-selected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
               (gobject.TYPE_PYOBJECT,) ),
           'webcam-caps-selected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                               (gobject.TYPE_PYOBJECT,))

           
           }


        
    def __init__(self, project_data, *args ) :
        gtk.Frame.__init__(self, *args )
        Signallable.__init__(self)
        Loggable.__init__(self)
        self.wcam_params = {}

        self.set_name('Page_webcam')
        self.project_data = project_data
        
        #
        # set default values
        #
        self.project_data['webcam_data'] = dict()
        
        #
        # Initailize widgets for webcam detection page 
        #
        self.webcam_props_widget = WebcamPropsWdg(self)
        
        self.webcam_props = WBCM_DTCT.WebcamProps(self.webcam_props_widget) 
        self.webcam_props.connect('caps-selected', self._caps_selected_cb)
        
        self._wcam_detect =  WBCM_DTCT.WebcamDetection()
        self.debug('Initialized')

        self.show_all()
        self.pipeline = None

    def prepare_webcam_detection(self) :
        """ prepare assistant page for the webcam detection page """
        
        # launch detection
        _nb = self._wcam_detect.detect_webcams()

        if _nb > 0 :
            _webcams = [ _device['name'] for _device in \
                    self._wcam_detect.webcam_devices]
            self.webcam_props_widget.display_webcams(_webcams)
            self.emit('webcams-detected', _webcams)
            self.webcam_props_widget.connect(
                    'webcam-changed', 
                    self._webcam_changed_cb) 
        else :
            self.info('No webcam detected')
            self.emit('webcam-not-detetcted')


    def active_webcam(self, device) :
        if self.pipeline is not None :
            try :
                self.pipeline.pause()
                self.render.deactivate()
            except :
                pass

        
        self.factory = WebcamSourceFactory()
        self.debug("%s", self.wcam_params)
        self.factory.webcam_params =  self.wcam_params
        self.pipeline = Pipeline()
        
        vsettings = StreamEncodeSettings(encoder="theoraenc")
        rsettings = RenderSettings(settings=[vsettings],
                                   muxer="oggmux")
        
        self.fakesink = FakeSinkFactory()
        rendersink = RenderSinkFactory(RenderFactory(settings=rsettings),
                                       self.fakesink)
        self.render = RenderAction()
        #self.pipeline.connect("eos", self._renderEOSCb)
        #self.pipeline.connect("error", self._renderErrorCb)
        self.pipeline.addAction(self.render)
        self.render.addConsumers(rendersink)
        self.render.addProducers(self.factory)
        self.render.activate() 
        self.pipeline.play()
        gobject.timeout_add(1000, self._deactive_pipeline)
        return self.factory
    
    def _deactive_pipeline(self) :
        self.debug('Deactive Pipeline')
        if self.pipeline is not None :
            try :
                print " release ."
                self.pipeline.stop()
                self.render.deactivate()
                self.render.setPipeline(None)
                self.pipeline.release()
                self.pipeline = None
                self.factory = None

            except :
                pass
        return False


    def _webcam_changed_cb(self, wdg, webcam) :
        self.debug('Rx webcam-changed : %s', webcam )
        self.webcam_props.clear_all()
        self.wcam_params = {}
        _selected_device = None
        
        for _device in self._wcam_detect.webcam_devices :
            if _device['name'] == webcam :
                _selected_device = _device
                break
        if _selected_device != None :
            # Emit webcam detetected
            self.debug('Emit webcam-selected device : %s ', _selected_device)
            self.project_data['webcam_data'] = _selected_device
            self.emit('webcam-selected', _selected_device)
            
            # prepare webcam param/caps detection
            self.wcam_params['wcam_driver'] = {
                    'element': _selected_device['v4ldriver'] ,
                    'properties' : {
                        'device':_selected_device['device-file'],
                        }
                    }
            _factory = self.active_webcam(self.wcam_params)
            self.webcam_props.prepare(_factory.webcam_caps)


        else :
            self.error('No selected device. Unknow %s', webcam)
    

    def _caps_selected_cb(self, webcam_props, caps) :
        self.project_data['webcam_data']['caps'] = caps
        self.debug('Rx caps-selected : %s', caps.to_string() )
        self.emit('webcam-caps-selected', caps)

    


    def _on_webcam_detect_complete(self, webcam_obj) :
        """ callback , executed when webcam detection is complete 
            Return the number of detected webcams"""
        _status = 0

        if webcam_obj != None and webcam_obj.webcam_devices != None:
            _vbox = self._vbox
            # firt clean childs on the vbox widget
            for _child in _vbox.get_children() : 
                _vbox.remove(_child)
            # loop on detected webcams 
            _rd_btn = None

            for (_wbcm_idx, _webcam ) in \
                    enumerate(webcam_obj.webcam_devices) :
                _rd_btn = gtk.RadioButton(
                        group =_rd_btn,
                        label = _webcam["name"] )
                _vbox.pack_start(_rd_btn)
                _rd_btn.show()
                
                #connect event 
                _rd_btn.connect(
                        "clicked",
                        self.on_webcam_radio_button_clicked,
                        _wbcm_idx, 
                        webcam_obj )
                
                # First set 
                if _wbcm_idx == 0 :
                    # select by default. 
                    self.project_data['webcam_data'] = \
                            webcam_obj.get_gst_best_input(_wbcm_idx) 
            
            self.label.set_text(_('Detected webcam(s)'))
            _status = len(webcam_obj.webcam_devices)
        else :
            self.project_data['webcam_data'] = None
            self.label.set_text(_('No webcam detected'))
        self.label.show()
        return _status
    
    def on_webcam_radio_button_clicked(self, widget, webcam_index, webcam_obj):
        """ callback for webcam rasdio button.
        Save info on selected webcam 
        """
        # when a webcam is selected select webcam data
        self.project_data['webcam_data'] = \
                webcam_obj.get_gst_best_input(webcam_index) 

        
if __name__ == '__main__' :
    import pygtk

    def init_log() :
        import os
        from pitivi.log import log
        enable_color = os.environ.get('PITIVI_DEBUG_NO_COLOR', '0') in ('', '0')
        log.init('PITIVI_DEBUG', enable_color)
    
    class Tester(Loggable) :

        def __init__(self) :
            Loggable.__init__(self)
            window = gtk.Window()
            window.connect('destroy', lambda w: gtk.main_quit())
            self.debug(' starting App') 
            _wdg = WebcamDetectionWidget(project_data = {})
            _wdg.connect('webcams-detected',self._webcam_detected_cb) 
            window.add(_wdg)
            window.show_all()
            
            _wdg.prepare_webcam_detection()

            return

        def _webcam_detected_cb(self, widget, data = None):
            self.debug('Rx webcam detected %s', data)

    init_log()
    Tester()
    gtk.main()


