#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
project_etree.py :
   Manage xml for project file
"""

# standard library imports
import os.path
import xml.etree.ElementTree as ET #python 2.5
from xml.parsers.expat import ExpatError

from pitivi.log.loggable import Loggable

# related third party imports
# N/A

# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.base.tools as LT
import luciole.constants as LCONST



class ProjectDico(dict):
    """ 
    Luciole project dicitionary : contains all porject data as dictionary 
    """
    
    #
    # CONSTANTS
    #

    # mandatory tags for webcam description 
    WEBCAM_DATA_TAGS = [ 'name',
                        #'device',
                        'device-file',
                        #'source_input',
                        'v4ldriver',
                        'caps'
                        #'height',
                        #'width',
                        #'framerate_list',
                        #'framerate_selected',
                        ]
    
    # default values for webcam
    WEBCAM_TAGS_DEFAULT_VALUES = {
            'framerate_list' : [(5,1) ],
            'framerate_selected' : (5,1)  ,
            }
                    
   
    def __init__(self):
        """ constructor : init a dict """
        dict.__init__(self)

class ProjectToFile(Loggable) :
    """ 
    Class for handling luciole projects : creation, open, save
    """
    __TEMPLATE_NAME = "project_template.xml"
    TEMPLATE_FILE = os.path.join(LCONST.TEMPLATE_DIR, 
                    __TEMPLATE_NAME)     
    def __init__(self):
        """ Constructor"""
        # init logger
        Loggable.__init__(self)
        self.debug("Entering ProjectToFile")
        # by default no project loaded
        self.__project_loaded = False
        self._project_xml = None
    
    def create(self, project_dico) :
        """ 
        create project hiararchy and XML structure
        returns the path to the xml projecfile 
        """
        project_dico['project_dir'] = \
                os.path.join(project_dico['project_dir'],
                        project_dico['project_name'])
        # create project dir and dirs hierarchy
        project_dico = self.__create_folders(project_dico)

        # update XML filename
        project_dico['xml_filename'] = project_dico['project_name']+'.xml'
        _project_path = os.path.join(
                                    project_dico['project_dir'],
                                    project_dico['xml_filename'])

        try : 
            self._project_xml = self._create_xml_file(project_dico)
        except LEXCEP.LucioException, _err :
            raise LEXCEP.LucioException, _err
        else :
            # create 3 empties list for rsuh capture and chrono
            project_dico['rush_images'] = [] 
            project_dico['capture_images'] = []
            project_dico['chrono_images'] = [] 
 
            self._save_to_xml_file(    
                                _project_path,
                                self._project_xml
                                )
        return _project_path


    def open(self, project_file_path) :
        """ open an existing project """
        self.debug(' try open : %s', project_file_path)
        _project = ProjectDico()
        if os.path.isfile(project_file_path) :
            #create  project dico
            _project['project_path'] = project_file_path
            _project['project_dir'] = os.path.split(project_file_path)[0]
            
            #parse XML project file
            try :
                self._project_xml = ET.parse(project_file_path)
            except ExpatError, err_value :
                _project = None
                _err =  "XML parse error in ", project_file_path, \
                        "\nerror :", err_value
                raise LEXCEP.LucioException, _err
                self.error(_err)
            else :
                _project['xml_filename'] = os.path.basename(project_file_path)
                # tmp dir nit in xml file but needed
                _project['tmdir'] = LCONST.TMP_DIR
                try : 
                    (_is_valid, _project_changed, _project) = \
                            self._scan_project_file(_project)
                except LEXCEP.LucioException, _err :
                    raise LEXCEP.LucioException, _err
                    self.error(_err)
                else :
                    self.__project_loaded = True
                    if _project_changed == True :
                        # save to file loaded project due to project change
                        # outside application 
                        self.save(_project)
                        self.debug("save project before open : due to project"
                                  " update according luciole versions")
                if _project != None : 
                    self.debug("Project %s is successfuly loaded", _project['xml_filename'])
                    self.__project_loaded = True
        else:
            # exception handling
            _err =  " invalid file name : ", project_file_path
            raise LEXCEP.LucioException, _err
        return (_is_valid, _project)

    def save(self, project_dico ) :
        """ save a project to XML file """
        self.debug (' Save props %s ', project_dico)
        # create XML object identical to create a new one
        self._project_xml = self._create_xml_file(project_dico)
        
        # put image name info ( rush, capture,chrono) in XML file
        self._project_xml = self.__create_xml_images_tag(
                                project_dico,
                                self._project_xml) 
        
        _project_path = os.path.join(
                                    project_dico['project_dir'],
                                    project_dico['xml_filename'])
        self._save_to_xml_file( _project_path, self._project_xml)
        return _project_path
        
    def save_as(self, new_project_dir, project_dico) :
        """ save as """
        self.debug (' Save props %s in %s', project_dico, new_project_dir)
        _project_path = None
        #memorise old rush_images dir 
        _src_rush_dir = os.path.join(  project_dico['project_dir'],
                                        project_dico['rush_dir'])

        project_dico['project_dir'] = new_project_dir
        # the new project name is the dir basename (last fir) of the path
        project_dico['project_name'] = os.path.basename(new_project_dir)
        project_dico['xml_filename'] = project_dico['project_name'] + '.xml'
         
        # 2. create folders hierarchy 
        project_dico = self.__create_folders(project_dico)
        
        # 3. copy rush images
        try :
            for _image_name in project_dico['rush_images'] :
                LT.copyf(os.path.join(_src_rush_dir, _image_name) , 
                        os.path.join(   project_dico['project_dir'],
                                        project_dico['rush_dir']) )
        except LEXCEP.LucioException, _err :
            _err =  " Impossible to copy %s  : ", _err
            raise LEXCEP.LucioException, _err
        else : 
            # 4 call save
            _project_path = self.save(project_dico)
        return _project_path

##############################################################################
##### PRIVATE  METHODS
##############################################################################
    def _save_to_xml_file(self, xml_file_path, project_xml):
        """ save an XML obect to file """
        # XML file indetation for readability
        self.__indent(project_xml.getroot())
        project_xml.write(xml_file_path,"UTF-8")

    def _scan_project_file(self, project) :
        """ scan xml project file """
        # by default a project is valid : 
        # validity is only available with WEBCAM hardtype 
        _is_valid = True 
        _project_changed = False

        # get path and dir data
        project['project_name'] = \
            self._project_xml.findtext("metas/projectName").strip()
        if project['project_dir'] != \
                self._project_xml.findtext("metas/projectPath").strip() :
            _project_changed = True 
        project['export_dir'] = \
                self._project_xml.findtext("metas/export_dir").strip()
        project['rush_dir'] = \
                self._project_xml.findtext("metas/rush_dir").strip()
        # tmp dir nit in xml file but needed
        project['tmdir'] = LCONST.TMP_DIR
        
        # get FPI
        # nbd@grape text or int value needed ?
        project['fpi'] = self._project_xml.findtext("metas/fpi").strip() 
        
        # get resize
        _elem =  self._project_xml.find("metas/resize")
        self.debug(' resize tag : %s', _elem)
        if _elem is None :
            # new tag added in luciole 0.9 : manage compatibilty
            project['resize'] = "yes"
            project['resize-mode'] = "PAL"
            _project_changed = True
            self.debug(' tag creation for resize')
        else :
            project['resize'] =  _elem.attrib["key"]
            project['resize-mode'] = _elem.attrib["mode"]



        # get hardware data
        _elem = self._project_xml.find("metas/hardtype")
        project['hardtype'] = int(_elem.attrib["id"])
        
        # get webcam data
        if project['hardtype'] == LCONST.WEBCAM :
            # loop on webcam datas
            et_webcam_data = self._project_xml.find("//hard_type_data")
            if et_webcam_data != None :
                # loop on webcam data and save on data dictionary
                webcam_data = {}
                for my_item in et_webcam_data.getchildren() :
                    # for framerate_selected and framerate_list tage : 
                    # evaluate to convert in tuple and list
                    # for others keep it as str
                    if my_item.tag == 'framerate_selected' or\
                       my_item.tag == 'framerate_list' :
                        webcam_data[my_item.tag] = \
                                eval(my_item.attrib.get("key"))
                    else :
                        webcam_data[my_item.tag] = my_item.attrib.get("key")

                # check webcam data validity 
                _is_valid = True
                _missing_tags = [] 
                for data in project.WEBCAM_DATA_TAGS :
                    if not webcam_data.has_key(data) :
                        _is_valid = False
                        _missing_tags.append(data)

                if _is_valid == False :
                    # not a valid project
                    for _missing_tag in _missing_tags :
                        if project.WEBCAM_TAGS_DEFAULT_VALUES.\
                                has_key(_missing_tag) :
                            webcam_data[_missing_tag] = \
                                project.\
                                WEBCAM_TAGS_DEFAULT_VALUES[_missing_tag]
                            _msg = "  missing tag : " , _missing_tag
                            self.debug(_msg)
                        else :
                            err = \
                            ( 'Invalid Project webcam data,'
                            ' tag %s is missing') % _missing_tag
                            raise LEXCEP.LucioException, err
                
                project['webcam_data'] = webcam_data
            else :
                # raise Error
                err =  "Invalid webcam data found "
                raise LEXCEP.LucioException, err


        project['rush_images'] = self._scan_images("rushData") 
        project['capture_images'] = self._scan_images("captureData") 
        project['chrono_images'] = self._scan_images("chronoData") 
      
        project = self.__check_rush_images(project)
        (_sound_project_changed, project['sound'] ) = \
            self._scan_sound_track()
        # update scan project tag    
        _project_changed = _project_changed or _sound_project_changed
        
        return (_is_valid, _project_changed, project)


    def _scan_images(self, img_type) :
        """ scan image tag and return image list """
        _list_out = []
        
        # get tag for image : param img_type 
        _data_tag = self._project_xml.find(img_type)

        for _image in _data_tag.getiterator('image') :
            # append  image name to list
            _list_out.append(_image.text.strip()) 
        
        return _list_out
    
    def _scan_sound_track(self) :
        """ scan for sound track """
        _dict = {}
        _project_changed = False
        _tag =  self._project_xml.find('sound')
        if _tag is not None :
            _attr = _tag.get('active')
            if _attr and _attr in ('yes','no') :
                _dict['active'] = _attr
            else :
                _err =  "invalid sound active attribute %s" % _attr
                raise LEXCEP.LucioException, _err

            _attr = _tag.get('stop_with_video')
            if _attr and _attr in ('yes','no') :
                _dict['stop_with_video'] = _attr
            else :
                _err =  "invalid sound stop_with_video attribute %s" % _attr
                raise LEXCEP.LucioException, _err
            
            _attr = _tag.get('name')
            if _attr and _attr == LCONST.SOUND_TRACK_NAME :
                _dict['name'] = _attr
            else :
                _err =  "invalid sound name attribute %s" % _attr
                raise LEXCEP.LucioException, _err
        else :
            # no sound tag found
            # possibly due to old luciole project ( 0.8 and later)
            # fill dict with no sound
            _dict['active'] = 'no'
            _dict['stop_with_video'] = 'no'
            _dict['name'] = LCONST.SOUND_TRACK_NAME
            _project_changed = True
        return (_project_changed, _dict)
            

        


    def __check_rush_images(self, project) :
        """ verify if all images in project really exist in dir. 
            if not remove it in project """
        _rush_dir =  os.path.join(
                project['project_dir'],
                project['rush_dir'] )
        _images_to_remove = []
        for _image_name in project['rush_images'] :
            # test if image exists
            img_path = os.path.join(_rush_dir, _image_name)
            if not os.path.exists(img_path) :
                msg =  "Not an image : ", _image_name
                self.info(msg)
                # image is not on rush dir
                # collect image to suppress  from rush_list
                _images_to_remove.append(_image_name)
        
        # now remove images in rush list
        for _image_name in _images_to_remove :
            project['rush_images'].remove(_image_name) 
               
            # remove also image ref in capture_images
            _nb_images = project['capture_images'].count(_image_name)
            if _nb_images > 0 :
                for _idx in xrange(_nb_images) : 
                    project['capture_images'].remove(_image_name)

            # remove also image ref in chrono_images
            _nb_images = project['chrono_images'].count(_image_name)
            if _nb_images > 0 : 
                for _idx in xrange(_nb_images) :
                    project['chrono_images'].remove(_image_name)
        
        return project





##############################################################################
##### STATIC METHODS
##############################################################################

    def __indent(elem, level=0):
        """ indent xml file """
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            for elem in elem:
                ProjectToFile.__indent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i
    __indent = staticmethod(__indent)

    def _create_xml_file(self, project_dico) :
        """ create the xml file for a project ans return it  """
        # load new xml file from template
        _project_xml = ET.parse(ProjectToFile.TEMPLATE_FILE)
        # update element tree file     
        _elem = _project_xml.find("metas/projectName")
        _elem.text =  project_dico['project_name']
        
        _elem = _project_xml.find("metas/projectPath")
        _elem.text = project_dico['project_dir']
       
        if project_dico.has_key('rush_dir') :
            _elem = _project_xml.find("metas/rush_dir")
            _elem.text = project_dico['rush_dir']

        if project_dico.has_key('export_dir') :
            _elem = _project_xml.find("metas/export_dir")
            _elem.text = project_dico['export_dir']

        # update fpi
        _elem = _project_xml.find("metas/fpi")
        if (project_dico['fpi']):
            _elem.text = str(project_dico['fpi'])
        else  :
            _elem.text = '5'
            #  nbd@grape :Warning Static method = Think nefor using logger
            self.warning("!!! No FPI selected, default value 5 frame per image  is set")
        
        # update resize
        _elem = _project_xml.find("metas/resize")
        _key = 'resize'
        if  _key in project_dico:
            if project_dico[_key] in ('yes', 'no') :
                _elem.attrib['key'] = project_dico[_key]
            else :
                self.warning(' key %s  in project dico has invalid value :%',
                        _key,
                        project_dico[_key])
        else :
            self.warning(' No %s key in project dico',_key)

        _key = 'resize-mode'
        if  _key in project_dico:
            if project_dico[_key] in ('FULL','PAL') :
                _elem.attrib['mode'] = project_dico[_key]
            else :
                self.warning(' key %s  in project dico has invalid value :%',
                        _key,
                        project_dico[_key])
        else :
            self.warning(' No %s key in project dico',_key)



        # update hardtype
        _elem = _project_xml.find("metas/hardtype")
        if (project_dico['hardtype']):
            _elem.attrib['id'] = str(project_dico['hardtype'])
            hard_type_desc = ET.SubElement(_elem, 'hard_type_desc') 
            hard_type_desc.text = \
                    LCONST.HARD_TYPE_NAME[project_dico['hardtype']]
            
            # in case of webcam save specific data
            if project_dico['hardtype'] == LCONST.WEBCAM and\
               project_dico['webcam_data'] != None : 
                hard_type_data = ET.SubElement(_elem, 'hard_type_data')
                # create new elements with same tage name as dictionary
                for _key, _val in project_dico['webcam_data'].iteritems() :
                    t_elem = ET.SubElement(hard_type_data, _key)
                    t_elem.attrib['key'] = str(_val)
        else :
            _elem.attrib['id'] = str(LCONST.FAKE)
            _elem.text = LCONST.HARD_TYPE_NAME[LCONST.FAKE]
            #  nbd@grape :Warning Static method = Think before using logger
            print " !!! No hardware selected, FAKE hardware selected"

        # update soundtrack
        _elem = _project_xml.find("sound")
        self._create_xml_sound_tags(_elem, project_dico)
        

        return _project_xml

    def _create_xml_sound_tags(tag, project_dico) :
        """ create and fill tags for sound track """
        if project_dico.has_key('sound') :
            _dict = project_dico['sound']
            
            _attr = 'active' 
            if _dict[_attr] and _dict[_attr] in ('yes','no') :
                tag.set(_attr, _dict[_attr])
            else :
                _err = 'Invalid %s attribute : %s' % (_attr, _dict[_attr])
                raise LEXCEP.LucioException, _err

            _attr = 'stop_with_video' 
            if _dict[_attr] and _dict[_attr] in ('yes','no') :
                tag.set(_attr, _dict[_attr])
            else :
                _err = 'Invalid %s attribute : %s' % (_attr, _dict[_attr])
                raise LEXCEP.LucioException, _err
            
            _attr = 'name' 
            if _dict[_attr] and _dict[_attr] in (LCONST.SOUND_TRACK_NAME) :
                tag.set(_attr, _dict[_attr])
            else :
                _err = 'Invalid %s attribute : %s' % (_attr, _dict[_attr])
                raise LEXCEP.LucioException, _err
        else :
            # No sound info on project
            # update it for consistency of project file
            tag.attrib['active'] = 'no'
            tag.attrib['stop_with_video'] = 'no'
            tag.attrib['name'] = LCONST.SOUND_TRACK_NAME
    _create_xml_sound_tags = staticmethod(_create_xml_sound_tags)

        
    def __create_xml_images_tag(project_dico, xml_file) :
        """ create image tag for rush, chrono and image """


        # rush images 
        __parent_image_tag = xml_file.find('rushData')
        __images_list =  project_dico['rush_images']
        __parent_image_tag = ProjectToFile.__make_tag_image_list(
                __parent_image_tag,
                __images_list)

        # capture images 
        __parent_image_tag = xml_file.find('captureData')
        __images_list =  project_dico['capture_images']
        __parent_image_tag = ProjectToFile.__make_tag_image_list(
                __parent_image_tag,
                __images_list)

        # chrono images 
        __parent_image_tag = xml_file.find('chronoData')
        __images_list =  project_dico['chrono_images']
        __parent_image_tag = ProjectToFile.__make_tag_image_list(
                __parent_image_tag,
                __images_list)

        # returm XML file with images
        return xml_file

    __create_xml_images_tag = staticmethod(__create_xml_images_tag)
    

    def __make_tag_image_list(tag , img_list) :
        """ create all the image tag elems and appen it to tag """
        for image in img_list :
            # create image element
            _elem = ET.Element('image')
            _elem.text = image
            # then append it in parent
            tag.append(_elem)            
        return tag
   
    __make_tag_image_list = staticmethod(__make_tag_image_list)

        
    def __create_folders(project_dico) : 
        """ create project dir and dirs hierarchy """
        project_dico['export_dir'] = 'export'
        project_dico['rush_dir'] = 'rush'
        project_dico['tmdir'] = LCONST.TMP_DIR
        _folder_to_create = [
                project_dico['project_dir'],
                os.path.join( project_dico['project_dir'],
                            project_dico['export_dir']),
                os.path.join( project_dico['project_dir'],
                              project_dico['rush_dir']),
                os.path.join( project_dico['project_dir'],
                             LCONST.TMP_DIR)
                ]
        try :
            for _folder in _folder_to_create :
                LT.mkdirs(_folder)
        except LEXCEP.LucioException, _err :
            _err =  "impossible to create %s : %s" % (_folder, _err)
            raise LEXCEP.LucioException, _err
        else :
            return project_dico

    __create_folders = staticmethod(__create_folders)





